searchData={"items":[{"type":"module","title":"JSV","doc":"JSV is a JSON Schema Validator.\n\nThis module is the main facade for the library.\n\nTo start validating schemas you will need to go through the following steps:\n\n1. [Obtain a schema](guides/schemas/defining-schemas.md). Schemas can be\n   defined in Elixir code, read from files, fetched remotely, _etc_.\n1. [Build a validation root](guides/build/build-basics.md) with `build/2` or\n   `build!/2`.\n1. [Validate the data](guides/validation/validation-basics.md).","ref":"JSV.html"},{"type":"module","title":"Example - JSV","doc":"Here is an example of the most simple way of using the library:\n\n```elixir\nschema = %{\n  type: :object,\n  properties: %{\n    name: %{type: :string}\n  },\n  required: [:name]\n}\n\nroot = JSV.build!(schema)\n\ncase JSV.validate(%{\"name\" => \"Alice\"}, root) do\n  {:ok, data} ->\n    {:ok, data}\n\n  # Errors can be turned into JSON compatible data structure to send them as an\n  # API response or for logging purposes.\n  {:error, validation_error} ->\n    {:error, JSON.encode!(JSV.normalize_error(validation_error))}\nend\n```\n\nIf you want to explore the different capabilities of the library, please refer\nto the guides provided in this documentation.","ref":"JSV.html#module-example"},{"type":"function","title":"JSV.build/2","doc":"Builds the schema as a `JSV.Root` schema for validation.","ref":"JSV.html#build/2"},{"type":"function","title":"Options - JSV.build/2","doc":"* `:resolver` - The `JSV.Resolver` behaviour implementation module to\n  retrieve schemas identified by an URL.\n\n  Accepts a `module`, a `{module, options}` tuple or a\n  list of those forms.\n\n  The options can be any term and will be given to the\n  `resolve/2` callback of the module.\n\n  The `JSV.Resolver.Embedded` and `JSV.Resolver.Internal`\n  will be automatically appended to support module-based\n  schemas and meta-schemas.\n\n  The default value is `[]`.\n\n* `:default_meta` (`t:String.t/0`) - The meta schema to use for resolved schemas that do not define a `\"$schema\"` property. The default value is `\"https://json-schema.org/draft/2020-12/schema\"`.\n\n* `:formats` - Controls the validation of strings with the `\"format\"` keyword.\n\n  * `nil` - Format validation is enabled if to the meta-schema uses the format assertion vocabulary.\n  * `true` - Enforces validation with the default validator modules.\n  * `false` - Disables all format validation.\n  * `[Module1, Module2,...]` (A list of modules) - Format validation is enabled and\n     will use those modules as validators instead of the default format validator modules.\n     The default format validator modules can be included back in the list manually,\n     see `default_format_validator_modules/0`.\n\n  > #### Formats are disabled by the default meta-schema {: .warning}\n  >\n  > The default value for this option is `nil` to respect\n  > the JSON Schema specification where format validation\n  > is enabled via vocabularies.\n  >\n  > The default meta-schemas for the latest drafts (example: `https://json-schema.org/draft/2020-12/schema`)\n  > do not enable format validation.\n  >\n  > You'll probably want this option to be set to `true`\n  > or a list of your own modules.\n\n  Worth noting, while this option does support providing your own formats,\n  the [official specification](https://json-schema.org/draft/2020-12/draft-bhutton-json-schema-validation-00#rfc.section.7.2.3)\n  recommends against it:\n\n  > Vocabularies do not support specifically declaring different value sets for keywords.\n  > Due to this limitation, and the historically uneven implementation of this keyword,\n  > it is RECOMMENDED to define additional keywords in a custom vocabulary rather than\n  > additional format attributes if interoperability is desired.\n\n  The default value is `nil`.\n\n* `:vocabularies` - Allows to redefine modules implementing vocabularies.\n\n  This option accepts a map with vocabulary URIs as keys and implementations as values.\n  The URIs are not fetched by JSV and does not need to point to anything specific.\n  For instance, vocabulary URIs in the standard Draft 2020-12 meta-schema point to\n  human-readable documentation.\n\n  The given implementations will only be used if the meta-schema used to build a validation root\n  actually declare those URIs in their `$vocabulary` keyword.\n\n  For instance, to redefine how the `type` keyword and other validation keywords are handled,\n  one should pass the following map:\n\n      %{\n        \"https://json-schema.org/draft/2020-12/vocab/validation\" => MyCustomModule\n      }\n\n  Modules must implement the `JSV.Vocabulary` behaviour.\n\n  Implementations can also be passed options by wrapping them in a tuple:\n\n      %{\n       \"https://json-schema.org/draft/2020-12/vocab/validation\" => {MyCustomModule, foo: \"bar\"}\n      }\n\n  The default value is `%{}`.","ref":"JSV.html#build/2-options"},{"type":"function","title":"JSV.build!/2","doc":"Same as `build/2` but raises on error. Errors are not normalized into a\n`JSV.BuildError` as `build/2` does.","ref":"JSV.html#build!/2"},{"type":"function","title":"JSV.build_add/2","doc":"Same as `build_add!/2` but rescues\nerrors and returns a result tuple.","ref":"JSV.html#build_add/2"},{"type":"function","title":"JSV.build_add!/2","doc":"Adds a schema to the build context.","ref":"JSV.html#build_add!/2"},{"type":"opaque","title":"JSV.build_context/0","doc":"","ref":"JSV.html#t:build_context/0"},{"type":"function","title":"JSV.build_init/1","doc":"Same as `build_init!/1` but rescues\nerrors and returns a result tuple.","ref":"JSV.html#build_init/1"},{"type":"function","title":"JSV.build_init!/1","doc":"Initializes a build context for controlled builds.\n\nSee `build/2` for options.","ref":"JSV.html#build_init!/1"},{"type":"function","title":"JSV.build_key/2","doc":"Same as `build_key!/2` but rescues\nerrors and returns a result tuple.","ref":"JSV.html#build_key/2"},{"type":"function","title":"JSV.build_key!/2","doc":"Builds the given reference or root schema.\n\nReturns the build context as well as a key, which is a pointer to the built\nschema.","ref":"JSV.html#build_key!/2"},{"type":"type","title":"JSV.build_opt/0","doc":"","ref":"JSV.html#t:build_opt/0"},{"type":"function","title":"JSV.default_format_validator_modules/0","doc":"Returns the list of format validator modules that are used when a schema is\nbuilt with format validation enabled and the `:formats` option to `build/2` is\n`true`.","ref":"JSV.html#default_format_validator_modules/0"},{"type":"function","title":"JSV.default_meta/0","doc":"Returns the default meta schema used when the `:default_meta` option is not\nset in `build/2`.\n\nCurrently returns \"https://json-schema.org/draft/2020-12/schema\".","ref":"JSV.html#default_meta/0"},{"type":"macro","title":"JSV.defcast/1","doc":"Enables a casting function in the current module, identified by its function\nname.","ref":"JSV.html#defcast/1"},{"type":"macro","title":"Example - JSV.defcast/1","doc":"```elixir\ndefmodule MyApp.Cast do\n  use JSV.Schema\n\n  defcast :to_integer\n\n  defp to_integer(data) when is_binary(data) do\n    case Integer.parse(data) do\n      {int, \"\"} -> {:ok, int}\n      _ -> {:error, \"invalid\"}\n    end\n  end\n\n  defp to_integer(_) do\n    {:error, \"invalid\"}\n  end\nend\n```\n\n    iex> schema = JSV.Schema.string() |> JSV.Schema.with_cast([\"Elixir.MyApp.Cast\", \"to_integer\"])\n    iex> root = JSV.build!(schema)\n    iex> JSV.validate(\"1234\", root)\n    {:ok, 1234}\n\nSee `defcast/3` for more information.","ref":"JSV.html#defcast/1-example"},{"type":"macro","title":"JSV.defcast/2","doc":"Enables a casting function in the current module, identified by a custom tag.","ref":"JSV.html#defcast/2"},{"type":"macro","title":"Example - JSV.defcast/2","doc":"```elixir\ndefmodule MyApp.Cast do\n  use JSV.Schema\n\n  defcast \"to_integer_if_string\", :to_integer\n\n  defp to_integer(data) when is_binary(data) do\n    case Integer.parse(data) do\n      {int, \"\"} -> {:ok, int}\n      _ -> {:error, \"invalid\"}\n    end\n  end\n\n  defp to_integer(_) do\n    {:error, \"invalid\"}\n  end\nend\n```\n\n    iex> schema = JSV.Schema.string() |> JSV.Schema.with_cast([\"Elixir.MyApp.Cast\", \"to_integer_if_string\"])\n    iex> root = JSV.build!(schema)\n    iex> JSV.validate(\"1234\", root)\n    {:ok, 1234}\n\nSee `defcast/3` for more information.","ref":"JSV.html#defcast/2-example"},{"type":"macro","title":"JSV.defcast/3","doc":"Defines a casting function in the calling module, and enables it for casting\ndata during validation.\n\nSee the [custom cast functions guide](cast-functions.html) to learn more about\ndefining your own cast functions.\n\nThis documentation assumes the following module is defined. Note that\n`JSV.Schema` provides several [predefined cast\nfunctions](JSV.Schema.html#schema-casters), including an [existing atom\ncast](JSV.Schema.html#string_to_existing_atom/0).\n\n```elixir\ndefmodule MyApp.Cast do\n  use JSV.Schema\n\n  defcast to_existing_atom(data) do\n    {:ok, String.to_existing_atom(data)}\n  rescue\n    ArgumentError -> {:error, \"bad atom\"}\n  end\n\n  def accepts_anything(data) do\n    {:ok, data}\n  end\nend\n```\n\nThis macro will define the `to_existing_atom/1` function in the calling\nmodule, and enable it to be referenced in the `jsv-cast` schema custom\nkeyword.\n\n    iex> MyApp.Cast.to_existing_atom(\"erlang\")\n    {:ok, :erlang}\n\n    iex> MyApp.Cast.to_existing_atom(\"not an existing atom\")\n    {:error, \"bad atom\"}\n\nIt will also define a zero arity function to get the cast information ready to\nbe included in a schema:\n\n    iex> MyApp.Cast.to_existing_atom()\n    [\"Elixir.MyApp.Cast\", \"to_existing_atom\"]\n\nThis is accepted by `JSV.Schema.with_cast/2`:\n\n    iex> JSV.Schema.with_cast(MyApp.Cast.to_existing_atom())\n    %JSV.Schema{\"jsv-cast\": [\"Elixir.MyApp.Cast\", \"to_existing_atom\"]}\n\nWith a `jsv-cast` property defined in a schema, data will be cast when the\nschema is validated:\n\n    iex> schema = JSV.Schema.string() |> JSV.Schema.with_cast(MyApp.Cast.to_existing_atom())\n    iex> root = JSV.build!(schema)\n    iex> JSV.validate(\"noreply\", root)\n    {:ok, :noreply}\n\n    iex> schema = JSV.Schema.string() |> JSV.Schema.with_cast(MyApp.Cast.to_existing_atom())\n    iex> root = JSV.build!(schema)\n    iex> {:error, %JSV.ValidationError{}} = JSV.validate([\"Elixir.NonExisting\"], root)\n\nIt is not mandatory to use the schema definition helpers. Raw schemas can\ncontain cast pointers too:\n\n    iex> schema = %{\n    ...>   \"type\" => \"string\",\n    ...>   \"jsv-cast\" => [\"Elixir.MyApp.Cast\", \"to_existing_atom\"]\n    ...> }\n    iex> root = JSV.build!(schema)\n    iex> JSV.validate(\"noreply\", root)\n    {:ok, :noreply}\n\nNote that for security reasons the cast pointer does not allow to call any\nfunction from the schema definition. A cast function MUST be enabled by\n`defcast/1`, `defcast/2` or `defcast/3`.\n\nThe `MyApp.Cast` example module above defines a `accepts_anything/1` function,\nbut the following schema will fail:\n\n    iex> schema = %{\n    ...>   \"type\" => \"string\",\n    ...>   \"jsv-cast\" => [\"Elixir.MyApp.Cast\", \"accepts_anything\"]\n    ...> }\n    iex> root = JSV.build!(schema)\n    iex> {:error, %JSV.ValidationError{errors: [%JSV.Validator.Error{kind: :\"bad-cast\"}]}} = JSV.validate(\"anything\", root)\n\nFinally, you can customize the name present in the `jsv-cast` property by\nusing a custom tag:\n\n```elixir\ndefcast \"my_custom_tag\", a_function_name(data) do\n  # ...\nend\n```\n\nMake sure to read the [custom cast functions guide](cast-functions.html)!","ref":"JSV.html#defcast/3"},{"type":"macro","title":"JSV.defschema/1","doc":"Defines a struct in the calling module where the struct keys are the\nproperties of the schema.\n\nThe given schema must define the `type` keyword as `object` and must define a\n`properties` map. That map can be empty to define a struct without any key.\nProperties keys must be given as atoms.\n\nThe `required` keyword is supported and must use atom keys as well.\n\nIf a default value is given in a property schema, it will be used as the\ndefault value for the corresponding struct key. Otherwise, the default value\nwill be `nil`. A default value is _not_ validated against the property schema\nitself.","ref":"JSV.html#defschema/1"},{"type":"macro","title":"Additional properties - JSV.defschema/1","doc":"Additional properties are allowed.\n\nIf your schema does not define `additionalProperties: false`, the validation\nwill accept a map with additional properties, but the keys will not be added\nto the resulting struct as it would be invalid.\n\nIf the `cast: false` option is given to `JSV.validate/3`, the additional\nproperties will be kept.","ref":"JSV.html#defschema/1-additional-properties"},{"type":"macro","title":"Property List Syntax - JSV.defschema/1","doc":"Alternatively, you can use a keyword list to define the properties where each\nproperty is defined as `{key, schema}`. The following rules apply:\n\n- All properties without a `default` value are automatically marked as\n  required and are enforced at the struct level.\n- The resulting schema will have `type: :object` set automatically.\n- The `title` of the schema is set as the last segment of the module name.\n\nThis provides a more concise way to define simple object schemas.","ref":"JSV.html#defschema/1-property-list-syntax"},{"type":"macro","title":"Examples - JSV.defschema/1","doc":"Given the following module definition:\n\n    defmodule MyApp.UserSchema do\n      import JSV\n\n      defschema %{\n        type: :object,\n        properties: %{\n          name: %{type: :string, default: \"\"},\n          age: %{type: :integer, default: 123}\n        }\n      }\n\n      # Or alternatively\n      defschema name: %{type: :string, default: \"\"},\n                age: %{type: :integer, default: 123}\n    end\n\nWe can get the struct with default values:\n\n    iex> %MyApp.UserSchema{}\n    %MyApp.UserSchema{name: \"\", age: 123}\n\n    iex> %MyApp.UserSchema{age: 999}\n    %MyApp.UserSchema{name: \"\", age: 999}\n\nAnd we can use the module as a schema:\n\n    iex> {:ok, root} = JSV.build(MyApp.UserSchema)\n    iex> data = %{\"name\" => \"Alice\"}\n    iex> JSV.validate(data, root)\n    {:ok, %MyApp.UserSchema{name: \"Alice\", age: 123}}\n\nAdditional properties are ignored:\n\n    iex> {:ok, root} = JSV.build(MyApp.UserSchema)\n    iex> data = %{\"name\" => \"Alice\", \"extra\" => \"hello!\"}\n    iex> JSV.validate(data, root)\n    {:ok, %MyApp.UserSchema{name: \"Alice\", age: 123}}\n\nDisabling struct casting with additional properties:\n\n    iex> {:ok, root} = JSV.build(MyApp.UserSchema)\n    iex> data = %{\"name\" => \"Alice\", \"extra\" => \"hello!\"}\n    iex> JSV.validate(data, root, cast: false)\n    {:ok, %{\"name\" => \"Alice\", \"extra\" => \"hello!\"}}\n\nA module can reference another module:\n\n    defmodule MyApp.CompanySchema do\n      require JSV\n\n      JSV.defschema(%{\n        type: :object,\n        properties: %{\n          name: %{type: :string},\n          owner: MyApp.UserSchema\n        }\n      })\n    end\n\n    iex> root = JSV.build!(MyApp.CompanySchema)\n    iex> data = %{\"name\" => \"Schemas Inc.\", \"owner\" => %{\"name\" => \"Alice\", \"age\" => 999}}\n    iex> JSV.validate(data, root)\n    {:ok, %MyApp.CompanySchema{\n      name: \"Schemas Inc.\",\n      owner: %MyApp.UserSchema{\n        name: \"Alice\",\n        age: 999\n      }\n    }}","ref":"JSV.html#defschema/1-examples"},{"type":"macro","title":"JSV.defschema/3","doc":"Defines a new module with a JSON Schema struct.\n\nThis macro is similar to `defschema/1` but it also takes a module name and\ndefines a nested module in the context where it is called. An optional\ndescription can be given, used as the `@moduledoc` and the description when a\nkeyword list of properties is given.\n\nThe module's struct will automatically `@derive` `Jason.Encoder` and\n`JSON.Encoder` if those modules are found during compilation.","ref":"JSV.html#defschema/3"},{"type":"macro","title":"Title and Description Behavior - JSV.defschema/3","doc":"When passing properties as a keyword list instead of a schema, the `title` and\n`description` parameters are automatically applied to the generated schema:\n\n- `title` is set from the module name (without outer module prefix if any)\n- `description` is set from the description parameter\n\nWhen passing a full schema map, the title and description from the parameters\nare not applied - the schema map is used as-is. Only the `description`\nparameter is used as the module's `@moduledoc`.","ref":"JSV.html#defschema/3-title-and-description-behavior"},{"type":"macro","title":"Examples - JSV.defschema/3","doc":"Basic module definition with keyword list:\n\n    defschema User,\n      name: string(),\n      age: integer(default: 0)\n\nModule with description using keyword list:\n\n    defschema User,\n              \"A user in the system\",\n              name: string(),\n              age: integer(default: 0)\n\nModule with full schema map:\n\n    defschema User,\n              \"User schema\",\n              %{\n                type: :object,\n                title: \"Custom Title\",\n                description: \"Custom Desc\",\n                properties: %{\n                  name: %{type: :string},\n                  age: %{type: :integer, default: 18}\n                },\n                required: [:name]\n              }","ref":"JSV.html#defschema/3-examples"},{"type":"macro","title":"Usage - JSV.defschema/3","doc":"The created module can be used like any struct:\n\n    %User{name: \"Alice\", age: 25}\n\nAnd as a JSON Schema for validation:\n\n    {:ok, root} = JSV.build(User)\n    JSV.validate(%{\"name\" => \"Bob\"}, root)\n    #=> {:ok, %User{name: \"Bob\", age: 0}}","ref":"JSV.html#defschema/3-usage"},{"type":"macro","title":"Module References - JSV.defschema/3","doc":"Modules can reference other modules in their properties:\n\n    defschema Address,\n              street: string(),\n              city: string()\n\n    defschema User,\n              name: string(),\n              address: Address\n\nUse `__MODULE__` for self-references:\n\n    defschema Category,\n              name: string(),\n              parent: optional(__MODULE__)","ref":"JSV.html#defschema/3-module-references"},{"type":"function","title":"JSV.error_schema/0","doc":"Returns the schema representing errors returned by `normalize_error/1`.\n\nBecause errors can be nested, the schema is recursive, so this function\nreturns a module based schema (a module name).","ref":"JSV.html#error_schema/0"},{"type":"type","title":"JSV.native_schema/0","doc":"A schema in native JSV/Elixir terms: maps with atoms, structs, and module.","ref":"JSV.html#t:native_schema/0"},{"type":"type","title":"JSV.normal_schema/0","doc":"A schema in a JSON-decoded form: Only maps with binary keys and\nbinary/number/boolean/nil values, or a boolean.\n\nThe name refers to the process of _normalization_. A `t:native_schema/0` can\nbe turned into a `t:normal_schema/0` with the help of\n`JSV.Schema.normalize/1`.","ref":"JSV.html#t:normal_schema/0"},{"type":"function","title":"JSV.normalize_error/2","doc":"Returns a JSON compatible represenation of a `JSV.ValidationError` struct.\n\nSee `JSV.ErrorFormatter.normalize_error/2` for options.\n\nWhen used without the `:atoms` keys option, a normalized error will correspond\nto the JSON schema returned by `error_schema/0`.","ref":"JSV.html#normalize_error/2"},{"type":"function","title":"JSV.resolver_chain/1","doc":"Normalizes a resolver implementation to a list of `{module, options}` and\nappends the default resolvers if they are not already present in the list.","ref":"JSV.html#resolver_chain/1"},{"type":"function","title":"Examples - JSV.resolver_chain/1","doc":"iex> JSV.resolver_chain(MyModule)\n    [{MyModule, []}, {JSV.Resolver.Embedded, []}, {JSV.Resolver.Internal, []}]\n\n    iex> JSV.resolver_chain([JSV.Resolver.Embedded, MyModule])\n    [{JSV.Resolver.Embedded, []}, {MyModule, []}, {JSV.Resolver.Internal, []}]\n\n    iex> JSV.resolver_chain([{JSV.Resolver.Embedded, []}, {MyModule, %{foo: :bar}}])\n    [{JSV.Resolver.Embedded, []}, {MyModule, %{foo: :bar}}, {JSV.Resolver.Internal, []}]","ref":"JSV.html#resolver_chain/1-examples"},{"type":"function","title":"JSV.to_root/2","doc":"Same as `to_root!/2` but rescues\nerrors and returns a result tuple.","ref":"JSV.html#to_root/2"},{"type":"function","title":"JSV.to_root!/2","doc":"Returns a root with all the validators from the build context and the given\n`root_key`. That key is used as the default entrypoint for validation when no\n`:key` option is passed to `validate/2`.","ref":"JSV.html#to_root!/2"},{"type":"function","title":"JSV.validate/3","doc":"Validates and casts the data with the given schema. The schema must be a\n`JSV.Root` struct generated with `build/2`.\n\n> #### This function returns cast data {: .info}\n>\n>\n> * If the `:cast_formats` option is enabled, string values may be transformed\n>   in other data structures. Refer to the \"Formats\" section of the\n>   [Validation guide](validation-basics.html#formats) for more information.\n> * The JSON Schema specification states that `123.0` is a valid integer. This\n>   function will return `123` instead. This may return invalid data for\n>   floats with very large integer parts. As always when dealing with JSON and\n>   big decimal or extremely precise numbers, use strings.","ref":"JSV.html#validate/3"},{"type":"function","title":"Options - JSV.validate/3","doc":"* `:cast` (`t:boolean/0`) - Enables calling generic cast functions on validation.\n\n  This is based on the `jsv-cast` JSON Schema custom keyword\n  and is typically used by `defschema/1`.\n\n  While it is on by default, some specific casting features are enabled\n  separately, see option `:cast_formats`.\n\n  The default value is `true`.\n\n* `:cast_formats` (`t:boolean/0`) - When enabled, format validators will return casted values,\n  for instance a `Date` struct instead of the date as string.\n\n  It has no effect when the schema was not built with formats enabled.\n\n  The default value is `false`.\n\n* `:key` (`t:term/0`) - When specified, the validation will start in the schema at the given key\n  instead of using the root schema.\n\n  The key must have been built and returned by `build_key!/2`. The validation\n  does not accept to validate any Ref or pointer in the schema.\n\n  This is useful when validating with a JSON document that contains schemas but\n  is not itself a schema.","ref":"JSV.html#validate/3-options"},{"type":"function","title":"JSV.validate!/3","doc":"","ref":"JSV.html#validate!/3"},{"type":"type","title":"JSV.validate_opt/0","doc":"","ref":"JSV.html#t:validate_opt/0"},{"type":"module","title":"JSV.Schema","doc":"This module defines a struct where all the supported keywords of the JSON\nschema specification are defined as keys. Text editors that can predict the\nstruct keys will make autocompletion available when writing schemas.","ref":"JSV.Schema.html"},{"type":"module","title":"Using in build - JSV.Schema","doc":"The `%JSV.Schema{}` struct can be given to `JSV.build/2`:\n\n    schema = %JSV.Schema{type: :integer}\n    JSV.build(schema, options())\n\nBecause Elixir structs always contain all their defined keys, writing a schema\nas `%JSV.Schema{type: :integer}` is actually defining the following:\n\n    %JSV.Schema{\n      type: :integer,\n      \"$id\": nil\n      additionalItems: nil,\n      additionalProperties: nil,\n      allOf: nil,\n      anyOf: nil,\n      contains: nil,\n      # etc...\n    }\n\nFor that reason, when giving a `%JSV.Schema{}` struct to `JSV.build/2`, any `nil` value is\nignored. The same behaviour can be defined for other struct by implementing\nthe `JSV.Normalizer.Normalize` protocol. Mere maps will keep their `nil`\nvalues.\n\nNote that `JSV.build/2` does not require `%JSV.Schema{}` structs, any map with binary or\natom keys is accepted.\n\nThis is also why the `%JSV.Schema{}` struct does not define the `const` keyword, because\n`nil` is a valid value for that keyword but there is no way to know if the\nvalue was omitted or explicitly defined as `nil`. To circumvent that you may\nuse the `enum` keyword or just use a regular map instead of this module's\nstruct:\n\n    %JSV.Schema{enum: [nil]}\n    # OR\n    %{const: nil}","ref":"JSV.Schema.html#module-using-in-build"},{"type":"module","title":"Functional helpers - JSV.Schema","doc":"This module also exports a small range of utility functions to ease writing\nschemas in a functional way.\n\nThis is mostly useful when generating schemas dynamically, or for shorthands.\n\nFor instance, instead of writing the following:\n\n    %Schema{\n      type: :object,\n      properties: %{\n        name: %Schema{type: :string, description: \"the name of the user\", minLength: 1},\n        age: %Schema{type: :integer, description: \"the age of the user\"}\n      },\n      required: [:name, :age]\n    }\n\nOne can write:\n\n    %Schema{\n      type: :object,\n      properties: %{\n        name: string(description: \"the name of the user\", minLength: 1),\n        age: integer(description: \"the age of the user\")\n      },\n      required: [:name, :age]\n    }\n\nThis is also useful when building schemas dynamically, as the helpers are\npipe-able one into another:\n\n    new()\n    |> props(\n      name: string(description: \"the name of the user\", minLength: 1),\n      age: integer(description: \"the age of the user\")\n    )\n    |> required([:name, :age])","ref":"JSV.Schema.html#module-functional-helpers"},{"type":"macro","title":"JSV.Schema.__using__/1","doc":"Use this module to define module-based schemas or schemas with the helpers\nAPI.\n\n* Imports struct and cast definitions from `JSV`.\n* Imports the `JSV.Schema.Helpers` module with the `string`, `integer`,\n  `enum`, _etc._ helpers.","ref":"JSV.Schema.html#__using__/1"},{"type":"macro","title":"Example - JSV.Schema.__using__/1","doc":"defmodule MySchema do\n      use JSV.Schema\n\n      defschema %{\n        type: :object,\n        properties: %{\n          foo: string(description: \"Some foo!\"),\n          bar: integer(minimum: 100) |> with_cast(__MODULE__,:hashid),\n          sub: props(sub_foo: string(), sub_bar: integer()) pp\n        }\n      }\n\n      defcast hashid(bar) do\n        {:ok, Hashids.decode!(bar, cipher())}\n      end\n    end","ref":"JSV.Schema.html#__using__/1-example"},{"type":"type","title":"JSV.Schema.attributes/0","doc":"","ref":"JSV.Schema.html#t:attributes/0"},{"type":"function","title":"JSV.Schema.combine/2","doc":"Merges two sets of attributes into a single map. Attributes can be a keyword\nlist or a map.","ref":"JSV.Schema.html#combine/2"},{"type":"function","title":"JSV.Schema.from_module/1","doc":"Calls the `json_schema/0` function on the given module, with a fallback to the\ndeprecated `schema/0` function if exported.","ref":"JSV.Schema.html#from_module/1"},{"type":"function","title":"JSV.Schema.merge/2","doc":"Merges the given key/values into the base schema. The merge is shallow and\nwill overwrite any pre-existing key.\n\nThis function is defined to work with the `JSV.Schema.Composer` API.\n\nThe resulting schema is always a map or a struct but the actual type depends\non the given base. It follows the followng rules:\n\n* **When the base type is a map or a struct, it is preserved**\n  - If the base is a `%JSV.Schema{}` struct, the `values` are merged in.\n  - If the base is another struct, the `values` a merged in. It will fail if\n    the struct does not define the overriden keys. No invalid struct is\n    generated.\n  - If the base is a mere map, it is **not** turned into a `%JSV.Schema{}` struct and the\n    `values` are merged in.\n\n* **Otherwise the base is cast to a `%JSV.Schema{}` struct**\n  - If the base is `nil`, the function returns a `%JSV.Schema{}` struct with the given\n    `values`.\n  - If the base is a keyword list, the list will be turned into a `%JSV.Schema{}` struct\n  and then the `values` are merged in.","ref":"JSV.Schema.html#merge/2"},{"type":"function","title":"Examples - JSV.Schema.merge/2","doc":"iex> JSV.Schema.merge(%JSV.Schema{description: \"base\"}, %{type: :integer})\n    %JSV.Schema{description: \"base\", type: :integer}\n\n    defmodule CustomSchemaStruct do\n      defstruct [:type, :description]\n    end\n\n    iex> JSV.Schema.merge(%CustomSchemaStruct{description: \"base\"}, %{type: :integer})\n    %CustomSchemaStruct{description: \"base\", type: :integer}\n\n    iex> JSV.Schema.merge(%CustomSchemaStruct{description: \"base\"}, %{format: :date})\n    ** (KeyError) struct CustomSchemaStruct does not accept key :format\n\n    iex> JSV.Schema.merge(%{description: \"base\"}, %{type: :integer})\n    %{description: \"base\", type: :integer}\n\n    iex> JSV.Schema.merge(nil, %{type: :integer})\n    %JSV.Schema{type: :integer}\n\n    iex> JSV.Schema.merge([description: \"base\"], %{type: :integer})\n    %JSV.Schema{description: \"base\", type: :integer}","ref":"JSV.Schema.html#merge/2-examples"},{"type":"type","title":"JSV.Schema.merge_base/0","doc":"","ref":"JSV.Schema.html#t:merge_base/0"},{"type":"function","title":"JSV.Schema.new/0","doc":"Returns a new empty schema.","ref":"JSV.Schema.html#new/0"},{"type":"function","title":"JSV.Schema.new/1","doc":"Returns a new schema with the given key/values.","ref":"JSV.Schema.html#new/1"},{"type":"function","title":"JSV.Schema.normalize/1","doc":"Normalizes a JSON schema with the help of `JSV.Normalizer.normalize/3` with\nthe following customizations:\n\n* `JSV.Schema` structs pairs where the value is `nil` will be removed.\n  `%JSV.Schema{type: :object, properties: nil, allOf: nil, ...}` becomes\n  `%{\"type\" => \"object\"}`.\n* Modules names that export a schema will be converted to a raw schema with a\n  reference to that module that can be resolved automatically by\n  `JSV.Resolver.Internal`.\n* Other atoms will be checked to see if they correspond to a module name that\n  exports a `json_schema/0` function.","ref":"JSV.Schema.html#normalize/1"},{"type":"function","title":"Examples - JSV.Schema.normalize/1","doc":"defmodule Elixir.ASchemaExportingModule do\n      def schema, do: %{}\n    end\n\n    iex> JSV.Schema.normalize(ASchemaExportingModule)\n    %{\"$ref\" => \"jsv:module:Elixir.ASchemaExportingModule\"}\n\n    defmodule AModuleWithoutExportedSchema do\n      def hello, do: \"world\"\n    end\n\n    iex> JSV.Schema.normalize(AModuleWithoutExportedSchema)\n    \"Elixir.AModuleWithoutExportedSchema\"","ref":"JSV.Schema.html#normalize/1-examples"},{"type":"type","title":"JSV.Schema.schema/0","doc":"","ref":"JSV.Schema.html#t:schema/0"},{"type":"type","title":"JSV.Schema.schema_data/0","doc":"","ref":"JSV.Schema.html#t:schema_data/0"},{"type":"function","title":"JSV.Schema.schema_module?/1","doc":"Returns whether the given atom is a module with a `schema/0` exported\nfunction.","ref":"JSV.Schema.html#schema_module?/1"},{"type":"type","title":"JSV.Schema.t/0","doc":"","ref":"JSV.Schema.html#t:t/0"},{"type":"function","title":"JSV.Schema.to_map/1","doc":"Returns the given `%JSV.Schema{}` as a map without keys containing\na `nil` value.","ref":"JSV.Schema.html#to_map/1"},{"type":"function","title":"JSV.Schema.with_cast/2","doc":"Includes the cast function in a schema. The cast function must be given as a\nlist with two items:\n\n* A module, as atom or string\n* A tag, as atom, string or integer.\n\nAtom arguments will be converted to string.","ref":"JSV.Schema.html#with_cast/2"},{"type":"function","title":"Examples - JSV.Schema.with_cast/2","doc":"iex> JSV.Schema.with_cast([MyApp.Cast, :a_cast_function])\n    %JSV.Schema{\"jsv-cast\": [\"Elixir.MyApp.Cast\", \"a_cast_function\"]}\n\n    iex> JSV.Schema.with_cast([MyApp.Cast, 1234])\n    %JSV.Schema{\"jsv-cast\": [\"Elixir.MyApp.Cast\", 1234]}\n\n    iex> JSV.Schema.with_cast([\"some_erlang_module\", \"custom_tag\"])\n    %JSV.Schema{\"jsv-cast\": [\"some_erlang_module\", \"custom_tag\"]}","ref":"JSV.Schema.html#with_cast/2-examples"},{"type":"module","title":"JSV.Schema.Helpers","doc":"Helpers to define schemas in plain Elixir code.","ref":"JSV.Schema.Helpers.html"},{"type":"function","title":"JSV.Schema.Helpers.all_of/2","doc":"Defines or merges onto a JSON Schema with `allOf: schemas`.","ref":"JSV.Schema.Helpers.html#all_of/2"},{"type":"function","title":"JSV.Schema.Helpers.any_of/2","doc":"Defines or merges onto a JSON Schema with `anyOf: schemas`.","ref":"JSV.Schema.Helpers.html#any_of/2"},{"type":"function","title":"JSV.Schema.Helpers.array_of/2","doc":"Defines or merges onto a JSON Schema with `type: :array` and `items: item_schema`.","ref":"JSV.Schema.Helpers.html#array_of/2"},{"type":"function","title":"JSV.Schema.Helpers.boolean/1","doc":"Defines or merges onto a JSON Schema with `type: :boolean`.","ref":"JSV.Schema.Helpers.html#boolean/1"},{"type":"function","title":"JSV.Schema.Helpers.const/2","doc":"Defines or merges onto a JSON Schema with `const: const`.","ref":"JSV.Schema.Helpers.html#const/2"},{"type":"function","title":"JSV.Schema.Helpers.date/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: :date`.","ref":"JSV.Schema.Helpers.html#date/1"},{"type":"function","title":"JSV.Schema.Helpers.datetime/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: :\"date-time\"`.","ref":"JSV.Schema.Helpers.html#datetime/1"},{"type":"function","title":"JSV.Schema.Helpers.email/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: :email`.","ref":"JSV.Schema.Helpers.html#email/1"},{"type":"function","title":"JSV.Schema.Helpers.enum/2","doc":"Defines or merges onto a JSON Schema with `enum: enum`.\n\nNote that in the JSON Schema specification, if the enum contains `1` then\n`1.0` is a valid value.","ref":"JSV.Schema.Helpers.html#enum/2"},{"type":"function","title":"JSV.Schema.Helpers.format/2","doc":"Defines or merges onto a JSON Schema with `format: format`.\n\nDoes **not** set the `type: :string` on the schema. Use `string_of/2` for a\nshortcut.","ref":"JSV.Schema.Helpers.html#format/2"},{"type":"function","title":"JSV.Schema.Helpers.integer/1","doc":"Defines or merges onto a JSON Schema with `type: :integer`.","ref":"JSV.Schema.Helpers.html#integer/1"},{"type":"function","title":"JSV.Schema.Helpers.neg_integer/1","doc":"Defines or merges onto a JSON Schema with `type: :integer` and `maximum: -1`.","ref":"JSV.Schema.Helpers.html#neg_integer/1"},{"type":"function","title":"JSV.Schema.Helpers.non_empty_string/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `minLength: 1`.","ref":"JSV.Schema.Helpers.html#non_empty_string/1"},{"type":"function","title":"JSV.Schema.Helpers.non_neg_integer/1","doc":"Defines or merges onto a JSON Schema with `type: :integer` and `minimum: 0`.","ref":"JSV.Schema.Helpers.html#non_neg_integer/1"},{"type":"function","title":"JSV.Schema.Helpers.number/1","doc":"Defines or merges onto a JSON Schema with `type: :number`.","ref":"JSV.Schema.Helpers.html#number/1"},{"type":"function","title":"JSV.Schema.Helpers.object/1","doc":"Defines or merges onto a JSON Schema with `type: :object`.\n\nSee the `props/2` function that accepts properties as a first argument.","ref":"JSV.Schema.Helpers.html#object/1"},{"type":"function","title":"JSV.Schema.Helpers.one_of/2","doc":"Defines or merges onto a JSON Schema with `oneOf: schemas`.","ref":"JSV.Schema.Helpers.html#one_of/2"},{"type":"function","title":"JSV.Schema.Helpers.optional/1","doc":"Marks a schema as optional when using the keyword list syntax with\n`JSV.defschema/1`.\n\nThis is useful for recursive module references where you want to avoid\ninfinite nesting requirements. When used in property list syntax with\n`defschema`, the property will not be marked as required.\n\n#### Example\n\n```\ndefschema name: string(),\n          parent: optional(MySelfReferencingModule)\n```","ref":"JSV.Schema.Helpers.html#optional/1"},{"type":"function","title":"JSV.Schema.Helpers.pos_integer/1","doc":"Defines or merges onto a JSON Schema with `type: :integer` and `minimum: 1`.","ref":"JSV.Schema.Helpers.html#pos_integer/1"},{"type":"type","title":"JSV.Schema.Helpers.properties/0","doc":"","ref":"JSV.Schema.Helpers.html#t:properties/0"},{"type":"function","title":"JSV.Schema.Helpers.properties/2","doc":"Defines or merges onto a JSON Schema with `properties: properties`.\n\nDoes **not** set the `type: :object` on the schema. Use `props/2` for a\nshortcut.","ref":"JSV.Schema.Helpers.html#properties/2"},{"type":"type","title":"JSV.Schema.Helpers.property_key/0","doc":"","ref":"JSV.Schema.Helpers.html#t:property_key/0"},{"type":"function","title":"JSV.Schema.Helpers.props/2","doc":"Defines or merges onto a JSON Schema with `type: :object` and `properties: properties`.","ref":"JSV.Schema.Helpers.html#props/2"},{"type":"function","title":"JSV.Schema.Helpers.ref/2","doc":"Defines or merges onto a JSON Schema with `$ref: ref`.\n\nReturns a schema referencing the given `ref`.\n\nA struct-based schema module name is not a valid reference. Modules should be\npassed directly where a schema (and not a `$ref`) is expected.\n\n#### Example\n\nFor instance to define a `user` property, this is valid:\n```\nprops(user: UserSchema)\n```\n\nThe following is invalid:\n```\n# Do not do this\nprops(user: ref(UserSchema))\n```","ref":"JSV.Schema.Helpers.html#ref/2"},{"type":"macro","title":"JSV.Schema.Helpers.sigil_SD/2","doc":"The Schema Description sigil.\n\nA sigil used to embed long texts in schemas descriptions. Replaces all\ncombinations of whitespace by a single whitespace and trims the string.\n\nIt does not support any modifier.\n\nNote that newlines are perfectly fine in schema descriptions, as they are\nsimply encoded as `\"\\n\"`. This sigil is intended for schemas that need to be\ncompressed because they are sent over the wire repeatedly (like in HTTP APIs\nor when working with LLMs).","ref":"JSV.Schema.Helpers.html#sigil_SD/2"},{"type":"macro","title":"Example - JSV.Schema.Helpers.sigil_SD/2","doc":"iex> ~SD\"\"\"\n    ...> This schema represents an elixir.\n    ...>\n    ...> An elixir is a potion with positive outcomes!\n    ...> \"\"\"\n    \"This schema represents an elixir. An elixir is a potion with positive outcomes!\"","ref":"JSV.Schema.Helpers.html#sigil_SD/2-example"},{"type":"function","title":"JSV.Schema.Helpers.string/1","doc":"Defines or merges onto a JSON Schema with `type: :string`.","ref":"JSV.Schema.Helpers.html#string/1"},{"type":"function","title":"JSV.Schema.Helpers.string_enum_to_atom/2","doc":"Defines or merges onto a JSON Schema with `type: :string`, `enum: enum` and `jsv-cast: JSV.Cast.string_to_atom()`.\n\nAccepts a list of atoms and returns a schema that validates  a string\nrepresentation of one of the given atoms.\n\nOn validation, a cast will be made to return the original atom value.\n\nThis is useful when dealing with enums that are represented as atoms in the\ncodebase, such as Oban job statuses or other Ecto enum types.\n\n    iex> schema = props(status: string_enum_to_atom([:executing, :pending]))\n    iex> root = JSV.build!(schema)\n    iex> JSV.validate(%{\"status\" => \"pending\"}, root)\n    {:ok, %{\"status\" => :pending}}\n\n> #### Does not support `nil` {: .warning}\n>\n> This function sets the `string` type on the schema. If `nil` is given in the\n> enum, the corresponding valid JSON value will be the `\"nil\"` string rather\n> than `null`. See `string_enum_to_atom_or_nil/2`.","ref":"JSV.Schema.Helpers.html#string_enum_to_atom/2"},{"type":"function","title":"JSV.Schema.Helpers.string_enum_to_atom_or_nil/2","doc":"Defines or merges onto a JSON Schema with `type: [:string, :null]`, `enum: enum` and `jsv-cast: JSV.Cast.string_to_atom_or_nil()`.\n\nLike `string_enum_to_atom/2` but also accepts the `null` JSON value as part of the\nenum.","ref":"JSV.Schema.Helpers.html#string_enum_to_atom_or_nil/2"},{"type":"function","title":"JSV.Schema.Helpers.string_of/2","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: format`.","ref":"JSV.Schema.Helpers.html#string_of/2"},{"type":"function","title":"JSV.Schema.Helpers.string_to_atom/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_atom()`.","ref":"JSV.Schema.Helpers.html#string_to_atom/1"},{"type":"function","title":"JSV.Schema.Helpers.string_to_boolean/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_boolean()`.","ref":"JSV.Schema.Helpers.html#string_to_boolean/1"},{"type":"function","title":"JSV.Schema.Helpers.string_to_existing_atom/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_existing_atom()`.","ref":"JSV.Schema.Helpers.html#string_to_existing_atom/1"},{"type":"function","title":"JSV.Schema.Helpers.string_to_float/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_float()`.","ref":"JSV.Schema.Helpers.html#string_to_float/1"},{"type":"function","title":"JSV.Schema.Helpers.string_to_integer/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_integer()`.","ref":"JSV.Schema.Helpers.html#string_to_integer/1"},{"type":"function","title":"JSV.Schema.Helpers.string_to_number/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_number()`.","ref":"JSV.Schema.Helpers.html#string_to_number/1"},{"type":"function","title":"JSV.Schema.Helpers.uri/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: :uri`.","ref":"JSV.Schema.Helpers.html#uri/1"},{"type":"function","title":"JSV.Schema.Helpers.uuid/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: :uuid`.","ref":"JSV.Schema.Helpers.html#uuid/1"},{"type":"function","title":"JSV.Schema.Helpers.~>/2","doc":"An alias for `JSV.Schema.combine/2`.","ref":"JSV.Schema.Helpers.html#~%3E/2"},{"type":"function","title":"Example - JSV.Schema.Helpers.~>/2","doc":"iex> object(description: \"a user\")\n    ...> ~> any_of([AdminSchema, CustomerSchema])\n    ...> ~> properties(foo: integer())\n    %{\n      type: :object,\n      description: \"a user\",\n      properties: %{foo: %{type: :integer}},\n      anyOf: [AdminSchema, CustomerSchema]\n    }","ref":"JSV.Schema.Helpers.html#~%3E/2-example"},{"type":"exception","title":"JSV.BuildError","doc":"A simple wrapper for errors returned from `JSV.build/2`.","ref":"JSV.BuildError.html"},{"type":"function","title":"JSV.BuildError.of/3","doc":"Wraps the given term as the `reason` in a `JSV.BuildError` struct.\n\nThe `action` should be a `{module, function, [arg1, arg2, ..., argN]}` tuple or\na mfa tuple whenever possible.","ref":"JSV.BuildError.html#of/3"},{"type":"behaviour","title":"JSV.FormatValidator","doc":"Behaviour for format validator implementations.\n\nSuch implementations must be given to `JSV.build/2` in the `:formats` option:\n\n    JSV.build!(raw_schema,\n      resolver: resolver,\n      formats: [MyModule | JSV.default_format_validator_modules()]\n    )\n\nEach given module is interrogated for format support when a schema is built.\nModules earlier in the list take precedence and if a format is found in the\nreturned value of the `c:supported_formats/0` callback, the module is selected\nfor compilation and no other module will be tried.\n\nA module can declare multiple formats.","ref":"JSV.FormatValidator.html"},{"type":"callback","title":"JSV.FormatValidator.applies_to_type?/2","doc":"Returns `true` if the given format should be used for the input data type.\n\nFor instance, the `\"date\"` format will not be validated if the input data is\nnot a string.\n\nNo validation should be done in this callback, only the _type_ of the data\nshould be considered.","ref":"JSV.FormatValidator.html#c:applies_to_type?/2"},{"type":"type","title":"JSV.FormatValidator.format/0","doc":"","ref":"JSV.FormatValidator.html#t:format/0"},{"type":"callback","title":"JSV.FormatValidator.supported_formats/0","doc":"Returns the list of the supported formats, as strings.","ref":"JSV.FormatValidator.html#c:supported_formats/0"},{"type":"callback","title":"JSV.FormatValidator.validate_cast/2","doc":"Receives the schema format as string, and the data.\n\nReturns a result tuple with data optionally casted to a more meaningful data\nstructure (for instance returning a `Date` struct instead of the string\nrepresentation of the date).","ref":"JSV.FormatValidator.html#c:validate_cast/2"},{"type":"module","title":"JSV.Root","doc":"Internal representation of a JSON schema built with `JSV.build/2`.\n\nThe original schema, in its string-keys form, can be retrieved in the `:raw`\nkey of the struct.","ref":"JSV.Root.html"},{"type":"type","title":"JSV.Root.t/0","doc":"","ref":"JSV.Root.html#t:t/0"},{"type":"exception","title":"JSV.ValidationError","doc":"The data structure returned by `JSV.validate/3` when validation fails.","ref":"JSV.ValidationError.html"},{"type":"function","title":"JSV.ValidationError.of/1","doc":"Wraps the given `JSV.Validator.Error` list into an `JSV.ValidationError`\nexception.","ref":"JSV.ValidationError.html#of/1"},{"type":"type","title":"JSV.ValidationError.t/0","doc":"","ref":"JSV.ValidationError.html#t:t/0"},{"type":"behaviour","title":"JSV.Resolver","doc":"A behaviour describing the implementation of a [guides/build/custom resolver.\nResolves remote resources when building a JSON schema.","ref":"JSV.Resolver.html"},{"type":"function","title":"JSV.Resolver.chain_of/2","doc":"Returns a new resolver, with the given behaviour implementations, and a\ndefault meta-schema URL to use with schemas that do not declare a `$schema`\nproperty.","ref":"JSV.Resolver.html#chain_of/2"},{"type":"function","title":"JSV.Resolver.fetch_resolved/2","doc":"Returns the raw schema identified by the given key if was previously resolved.","ref":"JSV.Resolver.html#fetch_resolved/2"},{"type":"function","title":"JSV.Resolver.fetch_vocabulary/2","doc":"Returns the $vocabulary property of a schema identified by its namespace.\n\nThe schema must have been resolved previously as a meta-schema (_i.e._ found\nin an $schema property of a resolved schema).","ref":"JSV.Resolver.html#fetch_vocabulary/2"},{"type":"type","title":"JSV.Resolver.resolvable/0","doc":"","ref":"JSV.Resolver.html#t:resolvable/0"},{"type":"function","title":"JSV.Resolver.resolve/2","doc":"Fetches the remote resource into the internal resolver cache and returns a new\nresolver with that updated cache.","ref":"JSV.Resolver.html#resolve/2"},{"type":"callback","title":"JSV.Resolver.resolve/2","doc":"Receives an URI and the options passed in the resolver tuple to `JSV.build/2`\nand returns a result tuple for a raw JSON schema map.\n\nReturning boolean schemas from resolvers is not supported. You may wrap the\nboolean value in a `$defs` or any other pointer as a workaround.\n\nSchemas will be normalized using `JSV.Schema.normalize/1`. If the resolver\nreturns schema that are already in JSON-decoded form (like a response body\nfrom an HTTP call) without atoms, module names or structs, the resolver\nimplementation can return `{:normal, map}` instead to skip the normalization.","ref":"JSV.Resolver.html#c:resolve/2"},{"type":"opaque","title":"JSV.Resolver.t/0","doc":"","ref":"JSV.Resolver.html#t:t/0"},{"type":"module","title":"JSV.Resolver.Embedded","doc":"A `JSV.Resolver` implementation that resolves known schemas shipped as part of\nthe `JSV` library.\n\nInternal URIs such as `jsv:module: ` are delegated to the\nJSV.Resolver.Internal resolver.","ref":"JSV.Resolver.Embedded.html"},{"type":"module","title":"Embedded schemas - JSV.Resolver.Embedded","doc":"* https://json-schema.org/draft/2020-12/schema\n* https://json-schema.org/draft/2020-12/meta/validation\n* https://json-schema.org/draft/2020-12/meta/unevaluated\n* https://json-schema.org/draft/2020-12/meta/meta-data\n* https://json-schema.org/draft/2020-12/meta/format-assertion\n* https://json-schema.org/draft/2020-12/meta/format-annotation\n* https://json-schema.org/draft/2020-12/meta/core\n* https://json-schema.org/draft/2020-12/meta/content\n* https://json-schema.org/draft/2020-12/meta/applicator\n* http://json-schema.org/draft-07/schema","ref":"JSV.Resolver.Embedded.html#module-embedded-schemas"},{"type":"function","title":"JSV.Resolver.Embedded.embedded_normalized_ids/0","doc":"Returns the list of meta schemas embedded in this resolver. The IDs are given\nin normalized form, _i.e._ URLs without fragments.","ref":"JSV.Resolver.Embedded.html#embedded_normalized_ids/0"},{"type":"module","title":"JSV.Resolver.Httpc","doc":"A `JSV.Resolver` implementation that will fetch the schemas from the web with\nthe help of the `:httpc` module.\n\nThis resolver requires a list of allowed URL prefixes to download from. It\nalso needs a proper JSON library to decode fetched schemas:\n\n* From Elixir 1.18, the `JSON` module is automatically available in the\n  standard library.\n* JSV can use [Jason](https://hex.pm/packages/jason) if listed in your\n  dependencies with the  `\"~> 1.0\"` requirement.\n* JSV also supports [Poison](https://hex.pm/packages/poison) with the `\"~> 6.0\n  or ~> 5.0\"` requirement.\n\nSchemas known by the `JSV.Resolver.Embedded` will be fetched from that module\ninstead of being fetched from the web. Allowed prefixes are not needed for\nthose schemas.","ref":"JSV.Resolver.Httpc.html"},{"type":"module","title":"Options - JSV.Resolver.Httpc","doc":"This resolver supports the following options:\n\n- `:allowed_prefixes` - This option is mandatory and contains the allowed\n  prefixes to download from. For security reasons, the prefixes should contain\n  at leath the first `/` of the URI path, like `https://example.com/` instead\n  of `https://example.com`.\n- `:cache_dir` - The path of a directory to cache downloaded resources. The\n  default value can be retrieved with `default_cache_dir/0` and is based on\n  `System.tmp_dir!/0`. The option also accepts `false` to disable that cache.\n  Note that there is no cache expiration mechanism.","ref":"JSV.Resolver.Httpc.html#module-options"},{"type":"module","title":"Example - JSV.Resolver.Httpc","doc":"resolver_opts = [allowed_prefixes: [\"https://my-company.org/schemas\"], cache_dir: \"_build/custom/dir\"]\n    JSV.build(schema, resolver: {JSV.Resolver.BuiltIn, resolver_opts})","ref":"JSV.Resolver.Httpc.html#module-example"},{"type":"function","title":"JSV.Resolver.Httpc.default_cache_dir/0","doc":"Returns the default directory used by the disk-based cache.","ref":"JSV.Resolver.Httpc.html#default_cache_dir/0"},{"type":"module","title":"JSV.Resolver.Internal","doc":"A `JSV.Resolver` implementation that resolves URIs pointing to the application\ncode base or JSV code base.\n\nA custom resolver implementation should delegate `jsv:` prefixed URIs to this\nmodule to enable support of the internal resolutions features.","ref":"JSV.Resolver.Internal.html"},{"type":"module","title":"Module based schemas - JSV.Resolver.Internal","doc":"This resolver will resolve `jsv:module:MODULE` URIs where `MODULE` is a string\nrepresentation of an Elixir module. Modules pointed at with such references\nMUST export a `json_schema/0` function that returns a normalized JSON schema\nwith binary keys and values.","ref":"JSV.Resolver.Internal.html#module-module-based-schemas"},{"type":"function","title":"JSV.Resolver.Internal.module_to_uri/1","doc":"Returns a JSV internal URI for the given module.","ref":"JSV.Resolver.Internal.html#module_to_uri/1"},{"type":"function","title":"Example - JSV.Resolver.Internal.module_to_uri/1","doc":"iex> module_to_uri(Inspect.Opts)\n    \"jsv:module:Elixir.Inspect.Opts\"","ref":"JSV.Resolver.Internal.html#module_to_uri/1-example"},{"type":"module","title":"JSV.Resolver.Local","doc":"This module allows to build `JSV.Resolver` implementations that resolves\nschemas based on disk based on their `$id` property. It is not itself a\n`JSV.Resolver` implementation.\n\nTo define your own local resolver, `use` this module by providing the\n`:source` option with a path to a directory, a path to a file, or a list of\npaths to directories and files:\n\n```elixir\nschemas_dir = \"priv/messaging/schemas\"\nother_schema = \"priv/users/user-schema.schema.json\"\n\ndefmodule MyApp.LocalResolver do\n  use JSV.Resolver.Local, source: [schemas_dir, other_schema]\nend\n```\n\nThe macro will read all `.json` files from the given sources and build an\nindex mapping the `$id` property of each schema to its JSON-deserialized\nvalue.\n\nFor convenience, nested lists are accepted. Duplicated files accepted as well\nbut not duplicates due to symlinks.","ref":"JSV.Resolver.Local.html"},{"type":"module","title":"Compilation and caching - JSV.Resolver.Local","doc":"Schemas are loaded directly into the generated module code. The module will\nrecompile everytime a loaded schema file is modified or deleted.\n\nRecompilation will also happen when new files are added in directories listed\nas sources.","ref":"JSV.Resolver.Local.html#module-compilation-and-caching"},{"type":"module","title":"Debugging - JSV.Resolver.Local","doc":"The `use JSV.Resolver.Local` also accepts the following options:\n\n* `:warn` - A `boolean` flag enabling compilation warnings when a `.json` file\n  cannot be read or loaded properly. Defaults to `true`.\n* `:debug` - A `boolean` flag enabling printouts of the loaded schemas on\n  compilation. Defaults to `false`.","ref":"JSV.Resolver.Local.html#module-debugging"},{"type":"module","title":"Example - JSV.Resolver.Local","doc":"The `\"priv/schemas/user-schema.schema.json\"` file contains the following JSON\ntext:\n\n```json\n{\n  \"$id\": \"myapp:user-0.0.1\",\n  \"type\": \"object\",\n  \"properties\": {\n    \"username\": {\n      \"type\": \"string\"\n    }\n  }\n}\n```\n\nThen this can be used as a source in your module.\n\n```elixir\ndefmodule MyApp.LocalResolver do\n  use JSV.Resolver.Local, source: \"priv/schemas\"\nend\n```\n\nYou can now build validation roots (`JSV.Root`) by referencing this user\nschema in `$ref` and by providing your resolver to the `JSV.build!/2`\nfunction:\n\n    iex> schema = %{\"$ref\" => \"myapp:user-0.0.1\"}\n    iex> root   = JSV.build!(schema, resolver: MyApp.LocalResolver)\n    iex> # Here we pass an invalid username (an integer)\n    iex> result = JSV.validate(%{\"username\" => 123}, root)\n    iex> match?({:error, %JSV.ValidationError{}}, result)\n    true\n\nYou can also directly fetch a schema from the defined module:\n\n    iex> MyApp.LocalResolver.resolve(\"myapp:user-0.0.1\")\n    {:ok,\n      %{\n        \"$id\" => \"myapp:user-0.0.1\",\n        \"type\" => \"object\",\n        \"properties\" => %{\"username\" => %{\"type\" => \"string\"}}\n      }}\n\n    iex> MyApp.LocalResolver.resolve(\"myapp:other\")\n    {:error, {:unknown_id, \"myapp:other\"}}\n\n\n\nRemember that schemas are identified and resolved by their `$id` property and\nnot their path.","ref":"JSV.Resolver.Local.html#module-example"},{"type":"behaviour","title":"JSV.Vocabulary","doc":"Behaviour for vocabulary implementation.\n\nA vocabulary module is used twice during the lifetime of a JSON schema:\n\n* When building a schema, the vocabulary module is given key/value pairs such\n  as `{\"type\", \"integer\"}` or `{\"properties\", map_of_schemas}` and must\n  consume or ignore the given keyword, storing custom validation data in an\n  accumulator for further use.\n* When validating a schema, the module is called with the data to validate and\n  the accumulated validation data to produce a validation result.","ref":"JSV.Vocabulary.html"},{"type":"macro","title":"JSV.Vocabulary.__using__/1","doc":"By using this module you will:\n\n* Declare that module as a behaviour\n* Import all macros from the module\n* Declare a `priority/0` function if the `:priority` option is provided.","ref":"JSV.Vocabulary.html#__using__/1"},{"type":"type","title":"JSV.Vocabulary.acc/0","doc":"Represents the accumulator initially returned by `c:init_validators/1` and\naccepted and returned by `c:handle_keyword/4`.\n\nThis accumulator is then given to `c:finalize_validators/1` and the\n`t:collection/0` type is used from there.","ref":"JSV.Vocabulary.html#t:acc/0"},{"type":"type","title":"JSV.Vocabulary.collection/0","doc":"Represents the final form of the collected keywords after the ultimate\ntransformation returned by `c:finalize_validators/1`.","ref":"JSV.Vocabulary.html#t:collection/0"},{"type":"macro","title":"JSV.Vocabulary.consume_keyword/1","doc":"Defines a `c:handle_keyword/4` callback that will return the current\naccumulator without changes, but preventing other vocabulary modules with\nlower priority (higher number) to be called with this keyword.\n\nThe keyword must be given in atom form.","ref":"JSV.Vocabulary.html#consume_keyword/1"},{"type":"type","title":"JSV.Vocabulary.data/0","doc":"","ref":"JSV.Vocabulary.html#t:data/0"},{"type":"callback","title":"JSV.Vocabulary.finalize_validators/1","doc":"","ref":"JSV.Vocabulary.html#c:finalize_validators/1"},{"type":"callback","title":"JSV.Vocabulary.format_error/3","doc":"","ref":"JSV.Vocabulary.html#c:format_error/3"},{"type":"callback","title":"JSV.Vocabulary.handle_keyword/4","doc":"","ref":"JSV.Vocabulary.html#c:handle_keyword/4"},{"type":"macro","title":"JSV.Vocabulary.ignore_any_keyword/0","doc":"Defines a `c:handle_keyword/4` callback that will return `:ignore` for any\ngiven value.\n\nGenerally used below `take_keyword/6`:\n\n      take_keyword :items, items when is_map(items), acc, builder, raw_schema do\n        # ...\n      end\n\n      ignore_any_keyword()","ref":"JSV.Vocabulary.html#ignore_any_keyword/0"},{"type":"macro","title":"JSV.Vocabulary.ignore_keyword/1","doc":"Defines a `c:handle_keyword/4` callback that will return `:ignore` for the\ngiven keyword. The keyword must be given in atom form.\n\nGenerally used below `take_keyword/6`:\n\n      take_keyword :items, items when is_map(items), acc, builder, raw_schema do\n        # ...\n      end\n\n      ignore_keyword(:additionalItems)\n      ignore_keyword(:prefixItems)","ref":"JSV.Vocabulary.html#ignore_keyword/1"},{"type":"callback","title":"JSV.Vocabulary.init_validators/1","doc":"","ref":"JSV.Vocabulary.html#c:init_validators/1"},{"type":"type","title":"JSV.Vocabulary.pair/0","doc":"","ref":"JSV.Vocabulary.html#t:pair/0"},{"type":"callback","title":"JSV.Vocabulary.priority/0","doc":"Returns the priority for applyting this module to the data.\n\nLower values (close to zero) will be applied first. You can think \"order\"\ninstead of \"priority\" but several modules can share the same priority value.\n\nThis can be useful to define vocabularies that depend on other vocabularies.\nFor instance, the `unevaluatedProperties` keyword needs \"properties\",\n\"patternProperties\", \"additionalProperties\" and \"allOf\", \"oneOf\", \"anyOf\",\n_etc._ to be ran before itself so it can lookup what has been evaluated.\n\nModules shipped in this library have priority of 100, 200, etc. up to 900 so\nyou can interleave your own vocabularies. Casting values to non-validable\nterms (such as structs or dates) should be done by vocabularies with a\npriority of 1000 and above.","ref":"JSV.Vocabulary.html#c:priority/0"},{"type":"function","title":"JSV.Vocabulary.take_integer/4","doc":"Adds the given integer to the list accumulator as a 2-tuple with the given\n`key`.\n\nFails if the value is not an integer. Floats with zero-fractional (as `123.0`)\nwill be accepted and converted to integer, as the JSON Schema spec dictates.","ref":"JSV.Vocabulary.html#take_integer/4"},{"type":"macro","title":"JSV.Vocabulary.take_keyword/6","doc":"An utility macro to ease declare vocabularies with atom keys.\n\nDefines the `c:handle_keyword/4` callback.\n\n**Important**\n\n- The keyword must be given in atom form.\n- The original goal was to allow atom keys and values everywhere. Schemas are\n  now converted to binary from before being built.\n- It is still useful to use this macro to the signature of the\n  `c:handle_keyword/4` callback can be changed easily without too much\n  refactoring.\n- Guards must be placed after the second argument:\n\n      take_keyword :items, items when is_map(items), acc, builder, raw_schema do\n        # ...\n      end","ref":"JSV.Vocabulary.html#take_keyword/6"},{"type":"function","title":"JSV.Vocabulary.take_number/4","doc":"Adds the given integer to the list accumulator as a 2-tuple with the given\n`key`.\n\nFails if the value is not a number.","ref":"JSV.Vocabulary.html#take_number/4"},{"type":"function","title":"JSV.Vocabulary.take_sub/4","doc":"Gives the sub raw schema to the builder and adds the build result in the list\naccumulator as a 2-tuple with the given `key`.","ref":"JSV.Vocabulary.html#take_sub/4"},{"type":"function","title":"JSV.Vocabulary.take_sub/5","doc":"Same as `take_sub/4` but uses a custom `path_segment` to append to the\n`schemaLocation` of the built subschema.","ref":"JSV.Vocabulary.html#take_sub/5"},{"type":"function","title":"JSV.Vocabulary.to_decimal/1","doc":"Casts the given integer to a %Decimal{} struct using `Decimal.from_float/1`\nfor floats.","ref":"JSV.Vocabulary.html#to_decimal/1"},{"type":"callback","title":"JSV.Vocabulary.validate/3","doc":"","ref":"JSV.Vocabulary.html#c:validate/3"},{"type":"macro","title":"JSV.Vocabulary.with_decimal/1","doc":"","ref":"JSV.Vocabulary.html#with_decimal/1"},{"type":"module","title":"JSV.Vocabulary.V202012.Applicator","doc":"Implementation for the `https://json-schema.org/draft/2020-12/vocab/applicator`\nvocabulary.","ref":"JSV.Vocabulary.V202012.Applicator.html"},{"type":"module","title":"JSV.Vocabulary.V202012.Content","doc":"Placeholder implementation for the\n`https://json-schema.org/draft/2020-12/vocab/content` vocabulary. No\nvalidation is performed.","ref":"JSV.Vocabulary.V202012.Content.html"},{"type":"module","title":"JSV.Vocabulary.V202012.Core","doc":"Implementation for the `https://json-schema.org/draft/2020-12/vocab/core`\nvocabulary.","ref":"JSV.Vocabulary.V202012.Core.html"},{"type":"module","title":"JSV.Vocabulary.V202012.Format","doc":"Implementation for the\n`https://json-schema.org/draft/2020-12/vocab/format-annotation` and\n`https://json-schema.org/draft/2020-12/vocab/format-assertion` vocabularies.","ref":"JSV.Vocabulary.V202012.Format.html"},{"type":"module","title":"JSV.Vocabulary.V202012.MetaData","doc":"Implementation for the `https://json-schema.org/draft/2020-12/vocab/meta-data`\nvocabulary.","ref":"JSV.Vocabulary.V202012.MetaData.html"},{"type":"module","title":"JSV.Vocabulary.V202012.Unevaluated","doc":"Implementation for the\n`https://json-schema.org/draft/2020-12/vocab/unevaluated` vocabulary.","ref":"JSV.Vocabulary.V202012.Unevaluated.html"},{"type":"module","title":"JSV.Vocabulary.V202012.Validation","doc":"Implementation for the\n`https://json-schema.org/draft/2020-12/vocab/validation` vocabulary.","ref":"JSV.Vocabulary.V202012.Validation.html"},{"type":"module","title":"JSV.Vocabulary.V7.Applicator","doc":"Implementation of the applicator vocabulary with draft 7 sepecifiticies.","ref":"JSV.Vocabulary.V7.Applicator.html"},{"type":"module","title":"JSV.Vocabulary.V7.Content","doc":"Implementation of the content vocabulary with draft 7 sepecifiticies. No\nvalidation is performed.","ref":"JSV.Vocabulary.V7.Content.html"},{"type":"module","title":"JSV.Vocabulary.V7.Core","doc":"Implementation of the core vocabulary with draft 7 sepecifiticies.","ref":"JSV.Vocabulary.V7.Core.html"},{"type":"module","title":"JSV.Vocabulary.V7.Format","doc":"Implementation of the format vocabulary with draft 7 sepecifiticies.","ref":"JSV.Vocabulary.V7.Format.html"},{"type":"function","title":"JSV.Vocabulary.V7.Format.finalize_validators/1","doc":"Callback implementation for  JSV.Vocabulary.finalize_validators/1 .","ref":"JSV.Vocabulary.V7.Format.html#finalize_validators/1"},{"type":"function","title":"JSV.Vocabulary.V7.Format.handle_keyword/4","doc":"Callback implementation for  JSV.Vocabulary.handle_keyword/4 .","ref":"JSV.Vocabulary.V7.Format.html#handle_keyword/4"},{"type":"function","title":"JSV.Vocabulary.V7.Format.init_validators/1","doc":"Callback implementation for  JSV.Vocabulary.init_validators/1 .","ref":"JSV.Vocabulary.V7.Format.html#init_validators/1"},{"type":"function","title":"JSV.Vocabulary.V7.Format.validate/3","doc":"Callback implementation for  JSV.Vocabulary.validate/3 .","ref":"JSV.Vocabulary.V7.Format.html#validate/3"},{"type":"module","title":"JSV.Vocabulary.V7.MetaData","doc":"Implementation of the meta-data vocabulary with draft 7 sepecifiticies.","ref":"JSV.Vocabulary.V7.MetaData.html"},{"type":"module","title":"JSV.Vocabulary.V7.Validation","doc":"Implementation of the validation vocabulary with draft 7 sepecifiticies.","ref":"JSV.Vocabulary.V7.Validation.html"},{"type":"function","title":"JSV.Vocabulary.V7.Validation.finalize_validators/1","doc":"Callback implementation for  JSV.Vocabulary.finalize_validators/1 .","ref":"JSV.Vocabulary.V7.Validation.html#finalize_validators/1"},{"type":"function","title":"JSV.Vocabulary.V7.Validation.handle_keyword/4","doc":"Callback implementation for  JSV.Vocabulary.handle_keyword/4 .","ref":"JSV.Vocabulary.V7.Validation.html#handle_keyword/4"},{"type":"function","title":"JSV.Vocabulary.V7.Validation.init_validators/1","doc":"Callback implementation for  JSV.Vocabulary.init_validators/1 .","ref":"JSV.Vocabulary.V7.Validation.html#init_validators/1"},{"type":"function","title":"JSV.Vocabulary.V7.Validation.validate/3","doc":"Callback implementation for  JSV.Vocabulary.validate/3 .","ref":"JSV.Vocabulary.V7.Validation.html#validate/3"},{"type":"module","title":"JSV.Codec","doc":"JSON encoder/decoder based on available implementation.\n\nFirst looks for Jason, then Poision, then JSON (available since Elixir 1.18).","ref":"JSV.Codec.html"},{"type":"function","title":"JSV.Codec.codec/0","doc":"Returns the module used for JSON encoding and decoding.","ref":"JSV.Codec.html#codec/0"},{"type":"function","title":"JSV.Codec.decode/1","doc":"Equivalent to `JSON.decode/1`.","ref":"JSV.Codec.html#decode/1"},{"type":"function","title":"JSV.Codec.decode!/1","doc":"Equivalent to `JSON.decode!/1`.","ref":"JSV.Codec.html#decode!/1"},{"type":"function","title":"JSV.Codec.encode!/1","doc":"Equivalent to `JSON.encode!/1`.","ref":"JSV.Codec.html#encode!/1"},{"type":"function","title":"JSV.Codec.encode_to_iodata!/1","doc":"Equivalent to `JSON.encode_to_iodata!/1`.","ref":"JSV.Codec.html#encode_to_iodata!/1"},{"type":"function","title":"JSV.Codec.format!/1","doc":"Equivalent to `JSON.encode!/1` with human readable indentation.\n\nRequires `Jason` or `Poison` for Elixir versions before 1.18 and OTP before\n27.0.","ref":"JSV.Codec.html#format!/1"},{"type":"function","title":"JSV.Codec.format_ordered!/2","doc":"Equivalent to `JSON.encode!/1` with map keys ordered according to the sorter\nfunction.\n\nThe sorter function will be called with two keys from the same map and should\nreturn `true` if the first argument precedes or is in the same place as the\nsecond one.\n\nRequires `Jason` for Elixir versions before 1.18 and OTP before 27.1.\n\nData must be normalized before ordered encoding. Passing structs will result\nin an error. Use `JSV.Normalizer.normalize/1` to normalize data in a\ncompatible form.","ref":"JSV.Codec.html#format_ordered!/2"},{"type":"function","title":"JSV.Codec.format_ordered_to_iodata!/2","doc":"Like `format_ordered!/2` but returns iodata instead of strings.","ref":"JSV.Codec.html#format_ordered_to_iodata!/2"},{"type":"function","title":"JSV.Codec.format_to_iodata!/1","doc":"Equivalent to `JSON.encode_to_iodata!/1` with human readable indentation.","ref":"JSV.Codec.html#format_to_iodata!/1"},{"type":"type","title":"JSV.Codec.key/0","doc":"","ref":"JSV.Codec.html#t:key/0"},{"type":"type","title":"JSV.Codec.key_sorter/0","doc":"","ref":"JSV.Codec.html#t:key_sorter/0"},{"type":"function","title":"JSV.Codec.supports_formatting?/0","doc":"","ref":"JSV.Codec.html#supports_formatting?/0"},{"type":"function","title":"JSV.Codec.supports_ordered_formatting?/0","doc":"","ref":"JSV.Codec.html#supports_ordered_formatting?/0"},{"type":"module","title":"JSV.Helpers.MapExt","doc":"Helpers to work with maps.","ref":"JSV.Helpers.MapExt.html"},{"type":"function","title":"JSV.Helpers.MapExt.from_struct_no_nils/1","doc":"Returns the given struct without its `:__struct__` key and any key whose value\nis `nil`.","ref":"JSV.Helpers.MapExt.html#from_struct_no_nils/1"},{"type":"module","title":"JSV.Helpers.Traverse","doc":"Helpers to work with nested generic data structures.","ref":"JSV.Helpers.Traverse.html"},{"type":"function","title":"JSV.Helpers.Traverse.postwalk/2","doc":"Updates a data structure in depth-first, post-order traversal.\n\nOperates like `postwalk/3` but without an accumulator. Handling continuations\nfor structs require to handle the accumulator, whose value MUST be `nil`.","ref":"JSV.Helpers.Traverse.html#postwalk/2"},{"type":"function","title":"JSV.Helpers.Traverse.postwalk/3","doc":"Updates a JSON-compatible data structure in depth-first, post-order traversal\nwhile carrying an accumulator.\n\nThe callback must return a `{new_value, new_acc}` tuple.\n\nNested data structures are given to the callback before their wrappers, and\nwhen the wrappers are called, their children are already updated.\n\nJSON-compatible only means that there are restrictions on map keys and struct\nvalues:\n\n* The callback function will be called for any key but will not traverse the\n  keys. For instance, with data such as `%{{x, y} => \"some city\"}`, the tuple\n  used as key will be passed as-is but the callback will not be called for\n  individual tuple elements.\n* Structs will be passed as `{:struct, value, continuation}`. The struct keys\n  and values will **NOT** have been traversed yet. To operate on the struct\n  keys you MUST call it manually. To respect the post-order of traversal, it\n  SHOULD be called before further transformation of the struct:\n\n      Traverse.postwalk(%MyStruct, [], fn\n        {:struct, my_struct, cont}, acc ->\n          {map, acc} = cont.(Map.from_struct(my_struct), acc)\n          {struct!(MyStruct, do_something_with_map(map)), acc}\n        {:val, ...} -> ...\n      end)\n\n  The continuation only accepts raw maps.\n\n* General data is accepted: tuples, pid, refs, etc. *","ref":"JSV.Helpers.Traverse.html#postwalk/3"},{"type":"type","title":"JSV.Helpers.Traverse.postwalk_cont/0","doc":"","ref":"JSV.Helpers.Traverse.html#t:postwalk_cont/0"},{"type":"type","title":"JSV.Helpers.Traverse.postwalk_item/0","doc":"","ref":"JSV.Helpers.Traverse.html#t:postwalk_item/0"},{"type":"function","title":"JSV.Helpers.Traverse.prewalk/2","doc":"Updates a data structure in depth-first, pre-order traversal.\n\nOperates like `prewalk/3` but without an accumulator.","ref":"JSV.Helpers.Traverse.html#prewalk/2"},{"type":"function","title":"JSV.Helpers.Traverse.prewalk/3","doc":"Updates a JSON-compatible data structure in depth-first, pre-order traversal\nwhile carrying an accumulator.\n\nThe callback must return a `{new_value, new_acc}` tuple.\n\n\nNested data structures are given iterated after the parent data has been given\nto the function. So it is possible to accept a container (map, list, tuple)\nand return another one from the callback before the children are iterated.\n\nJSON-compatible only means that there are restrictions on map keys and struct\nvalues:\n\n* The callback function will be called for any key but will not traverse the\n  keys. For instance, with data such as `%{{x, y} => \"some city\"}`, the tuple\n  used as key will be passed as-is but the callback will not be called for\n  individual tuple elements.\n* Structs are passed as a `{:struct, struct}` tuple.\n* General data is accepted: tuples, pid, refs, etc. *","ref":"JSV.Helpers.Traverse.html#prewalk/3"},{"type":"type","title":"JSV.Helpers.Traverse.prewalk_item/0","doc":"","ref":"JSV.Helpers.Traverse.html#t:prewalk_item/0"},{"type":"module","title":"JSV.Normalizer","doc":"A Normalizer for JSON data structures.","ref":"JSV.Normalizer.html"},{"type":"function","title":"JSV.Normalizer.default_on_general_atom/2","doc":"Default implementation for the `:on_general_atom` option of `normalize/2` and\n`normalize/3`. Transforms atoms to strings.","ref":"JSV.Normalizer.html#default_on_general_atom/2"},{"type":"type","title":"JSV.Normalizer.json_decoded_form/0","doc":"","ref":"JSV.Normalizer.html#t:json_decoded_form/0"},{"type":"function","title":"JSV.Normalizer.normalize/2","doc":"Returns the given term in a JSON-decoded form without general atoms or\nstructs.\n\nSee `normalize/3` for details and options.","ref":"JSV.Normalizer.html#normalize/2"},{"type":"function","title":"Examples - JSV.Normalizer.normalize/2","doc":"iex> JSV.Normalizer.normalize(%{name: :joe})\n    %{\"name\" => \"joe\"}\n\n    iex> JSV.Normalizer.normalize(%{\"name\" => :joe})\n    %{\"name\" => \"joe\"}\n\n    iex> JSV.Normalizer.normalize(%{\"name\" => \"joe\"})\n    %{\"name\" => \"joe\"}\n\n    iex> JSV.Normalizer.normalize(%{true: false})\n    %{\"true\" => false}\n\n    iex> JSV.Normalizer.normalize(%{specials: [true, false, nil]})\n    %{\"specials\" => [true, false, nil]}\n\nThis function is also used internally to normalize schemas.\n\n    iex> JSV.Normalizer.normalize(%JSV.Schema{title: nil, properties: nil})\n    %{}\n\n    iex> JSV.Normalizer.normalize(%JSV.Schema{type: :integer})\n    %{\"type\" => \"integer\"}\n\n    iex> JSV.Normalizer.normalize(%JSV.Schema{title: :\"My Schema\"})\n    %{\"title\" => \"My Schema\"}\n\nOther structs must implement the `JSV.Normalizer.Normalize` protocol.\n\n    iex> defimpl JSV.Normalizer.Normalize, for: Range do\n    iex>   def normalize(range), do: Map.from_struct(range)\n    iex> end\n    iex> JSV.Normalizer.normalize(1..10)\n    %{\"first\" => 1, \"last\" => 10, \"step\" => 1}","ref":"JSV.Normalizer.html#normalize/2-examples"},{"type":"function","title":"JSV.Normalizer.normalize/3","doc":"Returns the given term in a JSON-decoded form without general atoms or structs\nwith an accumulator.","ref":"JSV.Normalizer.html#normalize/3"},{"type":"function","title":"What is \"JSON-decoded\" form? - JSV.Normalizer.normalize/3","doc":"By that we mean that the returned data could have been returned by\n`JSON.decode!/1`:\n\n* Only maps, lists, strings, numbers and atoms.\n* Structs must implement the `JSV.Normalizer.Normalize` protocol.\n* `true`, `false` and `nil` will be kept as-is in all places except for map\n  keys.\n* `true`, `false` and `nil` as map keys will be converted to string.\n* Other atoms as values will be passed to the `:on_general_atom` callback (see\n  options).\n* Map keys must only be atoms, strings or numbers and will be converted to\n  strings.","ref":"JSV.Normalizer.html#normalize/3-what-is-json-decoded-form"},{"type":"function","title":"Options - JSV.Normalizer.normalize/3","doc":"* `:on_general_atom` - A callback accepting an atom found in the data and the\n  accumulator. Must return a JSON-decoded value.","ref":"JSV.Normalizer.html#normalize/3-options"},{"type":"function","title":"Examples - JSV.Normalizer.normalize/3","doc":"iex> on_general_atom = fn atom, acc ->\n    ...>   {\"found:#{atom}\", [atom|acc]}\n    ...> end\n    iex> opts = [on_general_atom: on_general_atom]\n    iex> acc_in = []\n    iex> JSV.Normalizer.normalize(%{an_atom: SomeAtom, a_string: \"hello\"}, acc_in, opts)\n    {%{\"an_atom\" => \"found:Elixir.SomeAtom\", \"a_string\" => \"hello\"}, [SomeAtom]}","ref":"JSV.Normalizer.html#normalize/3-examples"},{"type":"protocol","title":"JSV.Normalizer.Normalize","doc":"Protocol used by `JSV.Normalizer` to transform structs into JSON-compatible\ndata structures when normalizing a schema.\n\nWhen implementing this protocol you do not need to run any specific\nnormalization by yourself, but rather just return a map with all or a\nselection of keys. Keys can be atoms or binaries, and values will be\nnormalized recursively.","ref":"JSV.Normalizer.Normalize.html"},{"type":"function","title":"JSV.Normalizer.Normalize.normalize/1","doc":"","ref":"JSV.Normalizer.Normalize.html#normalize/1"},{"type":"type","title":"JSV.Normalizer.Normalize.t/0","doc":"All the types that implement this protocol.","ref":"JSV.Normalizer.Normalize.html#t:t/0"},{"type":"module","title":"JSV.Schema.Composer","doc":"This module contains a composable API to build schemas in a functionnal way.\n\nEvery function will return a schema and accepts an optional _first_ argument\nto merge onto, using `JSV.Schema.merge/2`.\n\nSee `JSV.Schema.Helpers` to work with a more \"presets\" oriented API.","ref":"JSV.Schema.Composer.html"},{"type":"module","title":"Example - JSV.Schema.Composer","doc":"iex> %JSV.Schema{}\n    ...> |> object()\n    ...> |> properties(foo: string())\n    ...> |> required([:foo])\n    %JSV.Schema{type: :object, properties: %{foo: %JSV.Schema{type: :string}}, required: [:foo]}","ref":"JSV.Schema.Composer.html#module-example"},{"type":"function","title":"JSV.Schema.Composer.all_of/2","doc":"Defines or merges onto a JSON Schema with `allOf: schemas`.","ref":"JSV.Schema.Composer.html#all_of/2"},{"type":"function","title":"JSV.Schema.Composer.any_of/2","doc":"Defines or merges onto a JSON Schema with `anyOf: schemas`.","ref":"JSV.Schema.Composer.html#any_of/2"},{"type":"function","title":"JSV.Schema.Composer.array_of/2","doc":"Defines or merges onto a JSON Schema with `type: :array` and `items: item_schema`.","ref":"JSV.Schema.Composer.html#array_of/2"},{"type":"function","title":"JSV.Schema.Composer.boolean/1","doc":"Defines or merges onto a JSON Schema with `type: :boolean`.","ref":"JSV.Schema.Composer.html#boolean/1"},{"type":"function","title":"JSV.Schema.Composer.date/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: :date`.","ref":"JSV.Schema.Composer.html#date/1"},{"type":"function","title":"JSV.Schema.Composer.datetime/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: :\"date-time\"`.","ref":"JSV.Schema.Composer.html#datetime/1"},{"type":"function","title":"JSV.Schema.Composer.email/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: :email`.","ref":"JSV.Schema.Composer.html#email/1"},{"type":"function","title":"JSV.Schema.Composer.format/2","doc":"Defines or merges onto a JSON Schema with `format: format`.\n\nDoes **not** set the `type: :string` on the schema. Use `string_of/2` for a\nshortcut.","ref":"JSV.Schema.Composer.html#format/2"},{"type":"function","title":"JSV.Schema.Composer.integer/1","doc":"Defines or merges onto a JSON Schema with `type: :integer`.","ref":"JSV.Schema.Composer.html#integer/1"},{"type":"function","title":"JSV.Schema.Composer.items/2","doc":"Defines or merges onto a JSON Schema with `items: item_schema`.\n\nDoes **not** set the `type: :array` on the schema. Use `array_of/2` for a\nshortcut.","ref":"JSV.Schema.Composer.html#items/2"},{"type":"function","title":"JSV.Schema.Composer.neg_integer/1","doc":"Defines or merges onto a JSON Schema with `type: :integer` and `maximum: -1`.","ref":"JSV.Schema.Composer.html#neg_integer/1"},{"type":"function","title":"JSV.Schema.Composer.non_empty_string/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `minLength: 1`.","ref":"JSV.Schema.Composer.html#non_empty_string/1"},{"type":"function","title":"JSV.Schema.Composer.non_neg_integer/1","doc":"Defines or merges onto a JSON Schema with `type: :integer` and `minimum: 0`.","ref":"JSV.Schema.Composer.html#non_neg_integer/1"},{"type":"function","title":"JSV.Schema.Composer.number/1","doc":"Defines or merges onto a JSON Schema with `type: :number`.","ref":"JSV.Schema.Composer.html#number/1"},{"type":"function","title":"JSV.Schema.Composer.object/1","doc":"Defines or merges onto a JSON Schema with `type: :object`.\n\nSee `props/2` to define the properties as well.","ref":"JSV.Schema.Composer.html#object/1"},{"type":"function","title":"JSV.Schema.Composer.one_of/2","doc":"Defines or merges onto a JSON Schema with `oneOf: schemas`.","ref":"JSV.Schema.Composer.html#one_of/2"},{"type":"function","title":"JSV.Schema.Composer.pos_integer/1","doc":"Defines or merges onto a JSON Schema with `type: :integer` and `minimum: 1`.","ref":"JSV.Schema.Composer.html#pos_integer/1"},{"type":"type","title":"JSV.Schema.Composer.properties/0","doc":"","ref":"JSV.Schema.Composer.html#t:properties/0"},{"type":"function","title":"JSV.Schema.Composer.properties/2","doc":"Defines or merges onto a JSON Schema with `properties: properties`.\n\nDoes **not** set the `type: :object` on the schema. Use `props/2` for a\nshortcut.","ref":"JSV.Schema.Composer.html#properties/2"},{"type":"type","title":"JSV.Schema.Composer.property_key/0","doc":"","ref":"JSV.Schema.Composer.html#t:property_key/0"},{"type":"function","title":"JSV.Schema.Composer.props/2","doc":"Defines or merges onto a JSON Schema with `type: :object` and `properties: properties`.","ref":"JSV.Schema.Composer.html#props/2"},{"type":"function","title":"JSV.Schema.Composer.ref/2","doc":"Defines or merges onto a JSON Schema with `$ref: ref`.\n\nA struct-based schema module name is not a valid reference. Modules should be\npassed directly where a schema (and not a `$ref`) is expected.\n\n#### Example\n\nFor instance to define a `user` property, this is valid:\n```\nprops(user: UserSchema)\n```\n\nThe following is invalid:\n```\n# Do not do this\nprops(user: ref(UserSchema))\n```","ref":"JSV.Schema.Composer.html#ref/2"},{"type":"function","title":"JSV.Schema.Composer.required/2","doc":"Defines a JSON Schema with `required: keys` or adds the given `keys` if the\n[base schema](JSV.Schema.html#merge/2) already has a `:required`\ndefinition.\n\nExisting required keys are preserved.","ref":"JSV.Schema.Composer.html#required/2"},{"type":"function","title":"Examples - JSV.Schema.Composer.required/2","doc":"iex> JSV.Schema.required(%{}, [:a, :b])\n    %{required: [:a, :b]}\n\n    iex> JSV.Schema.required(%{required: nil}, [:a, :b])\n    %{required: [:a, :b]}\n\n    iex> JSV.Schema.required(%{required: [:c]}, [:a, :b])\n    %{required: [:a, :b, :c]}\n\n    iex> JSV.Schema.required(%{required: [:a]}, [:a])\n    %{required: [:a, :a]}\n\nUse `JSV.Schema.merge/2` to replace existing required keys.\n\n    iex> JSV.Schema.merge(%{required: [:a, :b, :c]}, required: [:x, :y, :z])\n    %{required: [:x, :y, :z]}","ref":"JSV.Schema.Composer.html#required/2-examples"},{"type":"function","title":"JSV.Schema.Composer.string/1","doc":"Defines or merges onto a JSON Schema with `type: :string`.","ref":"JSV.Schema.Composer.html#string/1"},{"type":"function","title":"JSV.Schema.Composer.string_of/2","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: format`.","ref":"JSV.Schema.Composer.html#string_of/2"},{"type":"function","title":"JSV.Schema.Composer.string_to_atom/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_atom()`.","ref":"JSV.Schema.Composer.html#string_to_atom/1"},{"type":"function","title":"JSV.Schema.Composer.string_to_atom_enum/2","doc":"Defines or merges onto a JSON Schema with `type: :string`, `enum: enum` and `jsv-cast: JSV.Cast.string_to_atom()`.\n\nAccepts a list of atoms and validates that a given value is a string\nrepresentation of one of the given atoms.\n\nOn validation, a cast will be made to return the original atom value.\n\nThis is useful when dealing with enums that are represented as atoms in the\ncodebase, such as Oban job statuses or other Ecto enum types.\n\n    iex> schema = JSV.Schema.props(status: JSV.Schema.Composer.string_to_atom_enum([:executing, :pending]))\n    iex> root = JSV.build!(schema)\n    iex> JSV.validate(%{\"status\" => \"pending\"}, root)\n    {:ok, %{\"status\" => :pending}}\n\n> #### Does not support `nil` {: .warning}\n>\n> This function sets the `string` type on the schema. If `nil` is given in the\n> enum, the corresponding valid JSON value will be the `\"nil\"` string rather\n> than `null`","ref":"JSV.Schema.Composer.html#string_to_atom_enum/2"},{"type":"function","title":"JSV.Schema.Composer.string_to_boolean/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_boolean()`.","ref":"JSV.Schema.Composer.html#string_to_boolean/1"},{"type":"function","title":"JSV.Schema.Composer.string_to_existing_atom/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_existing_atom()`.","ref":"JSV.Schema.Composer.html#string_to_existing_atom/1"},{"type":"function","title":"JSV.Schema.Composer.string_to_float/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_float()`.","ref":"JSV.Schema.Composer.html#string_to_float/1"},{"type":"function","title":"JSV.Schema.Composer.string_to_integer/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_integer()`.","ref":"JSV.Schema.Composer.html#string_to_integer/1"},{"type":"function","title":"JSV.Schema.Composer.string_to_number/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `jsv-cast: JSV.Cast.string_to_number()`.","ref":"JSV.Schema.Composer.html#string_to_number/1"},{"type":"function","title":"JSV.Schema.Composer.uri/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: :uri`.","ref":"JSV.Schema.Composer.html#uri/1"},{"type":"function","title":"JSV.Schema.Composer.uuid/1","doc":"Defines or merges onto a JSON Schema with `type: :string` and `format: :uuid`.","ref":"JSV.Schema.Composer.html#uuid/1"},{"type":"module","title":"JSV.BooleanSchema","doc":"Represents a boolean schema. Boolean schemas accept or reject any data\naccording to their boolean value.\n\nThis is very often used with the `additionalProperties` keyword.","ref":"JSV.BooleanSchema.html"},{"type":"function","title":"JSV.BooleanSchema.of/2","doc":"Returns a `JSV.BooleanSchema` struct wrapping the given boolean.","ref":"JSV.BooleanSchema.html#of/2"},{"type":"type","title":"JSV.BooleanSchema.t/0","doc":"","ref":"JSV.BooleanSchema.html#t:t/0"},{"type":"module","title":"JSV.Builder","doc":"Internal logic to build raw schemas into `JSV.Root` structs.","ref":"JSV.Builder.html"},{"type":"function","title":"JSV.Builder.add_schema!/3","doc":"Adds a schema under the given key to the build, so that key or references to\nthis schema or its subschemas become buildable.","ref":"JSV.Builder.html#add_schema!/3"},{"type":"function","title":"JSV.Builder.build!/3","doc":"Builds the given root schema or reference into the given validators.","ref":"JSV.Builder.html#build!/3"},{"type":"function","title":"JSV.Builder.build_sub!/3","doc":"Builds a subschema. Called from vocabulary modules to build nested schemas\nsuch as in properties, if/else, items, etc.","ref":"JSV.Builder.html#build_sub!/3"},{"type":"type","title":"JSV.Builder.buildable/0","doc":"","ref":"JSV.Builder.html#t:buildable/0"},{"type":"function","title":"JSV.Builder.ensure_resolved!/2","doc":"Ensures that the remote resource that the given reference or key points to is\nfetched in the builder internal cache","ref":"JSV.Builder.html#ensure_resolved!/2"},{"type":"function","title":"JSV.Builder.fail/3","doc":"","ref":"JSV.Builder.html#fail/3"},{"type":"function","title":"JSV.Builder.fetch_resolved!/2","doc":"Returns the raw schema identified by the given key. Use `ensure_resolved/2`\nbefore if the resource may not have been fetched.","ref":"JSV.Builder.html#fetch_resolved!/2"},{"type":"function","title":"JSV.Builder.new/1","doc":"Returns a new builder. Builders are not reusable ; a fresh builder must be\nmade for each different root schema.","ref":"JSV.Builder.html#new/1"},{"type":"type","title":"JSV.Builder.path_segment/0","doc":"","ref":"JSV.Builder.html#t:path_segment/0"},{"type":"type","title":"JSV.Builder.resolvable/0","doc":"","ref":"JSV.Builder.html#t:resolvable/0"},{"type":"function","title":"JSV.Builder.stage_build/2","doc":"Adds a new key to be built later. A key is generatlly derived from a\nreference.","ref":"JSV.Builder.html#stage_build/2"},{"type":"type","title":"JSV.Builder.t/0","doc":"","ref":"JSV.Builder.html#t:t/0"},{"type":"function","title":"JSV.Builder.vocabulary_enabled?/2","doc":"","ref":"JSV.Builder.html#vocabulary_enabled?/2"},{"type":"module","title":"JSV.ErrorFormatter","doc":"Error formatting helpers.\n\nErrors are grouped by:\n\n* Instance location: the bit of data that was invalidated\n* Schema location: the part of the schema that invalidated it\n* Evaluation path: the path followed from the root to this schema location","ref":"JSV.ErrorFormatter.html"},{"type":"type","title":"JSV.ErrorFormatter.error_unit/0","doc":"","ref":"JSV.ErrorFormatter.html#t:error_unit/0"},{"type":"function","title":"JSV.ErrorFormatter.format_data_path/1","doc":"","ref":"JSV.ErrorFormatter.html#format_data_path/1"},{"type":"type","title":"JSV.ErrorFormatter.keyword_error/0","doc":"","ref":"JSV.ErrorFormatter.html#t:keyword_error/0"},{"type":"function","title":"JSV.ErrorFormatter.normalize_error/2","doc":"Returns a JSON-able version of the errors contained in the ValidationError.\n\nThis is generatlly useful to generate HTTP API responses or message broker\nresponses.","ref":"JSV.ErrorFormatter.html#normalize_error/2"},{"type":"function","title":"Options - JSV.ErrorFormatter.normalize_error/2","doc":"* `:sort` - Controls the sort direction. Errors are sorted by `instanceLocation`. The default value is `:desc`.\n\n* `:keys` - Define the type of the keys in the normalized errors maps.\n\n  While truly \"normalized\" JSON data should not have atom keys,\n  this option defaults to :atoms for backward compatibility reasons.","ref":"JSV.ErrorFormatter.html#normalize_error/2-options"},{"type":"type","title":"JSV.ErrorFormatter.normalize_opt/0","doc":"","ref":"JSV.ErrorFormatter.html#t:normalize_opt/0"},{"type":"type","title":"JSV.ErrorFormatter.raw_path/0","doc":"","ref":"JSV.ErrorFormatter.html#t:raw_path/0"},{"type":"function","title":"JSV.ErrorFormatter.valid_annot/2","doc":"Returns an output unit with `valid: true` for the given\n`JSV.Validator`. This can be substitued to an Error struct in the\nnested details of an error. Mostly used to show multiple validated schemas\nwith `:oneOf`.","ref":"JSV.ErrorFormatter.html#valid_annot/2"},{"type":"module","title":"JSV.Key","doc":"Helpers around the different key formats used in the resolver, builder and\nvalidator states to index sub schemas, referenced schemas, anchor or meta\nschemas.\n\nFor instance:\n\n* We have a `{\"$ref\": \"http://some-schema/#$defs/order\"}` JSON schema.\n\n* The builder will build the `$ref` keyword as a key: `{:pointer,\n  \"http://some-schema/\", [\"$defs\",\"order\"]}`.\n\n* The builder, via the resolver, will fetch `http://some-schema/`, store it\n  locally and build validators. Those validators will be stored under the same\n  key (`{:pointer, \"http://some-schema/\", [\"$defs\",\"order\"]}`) in the root\n  schema.\n\n* When the validator will validate the reference, it will fetch that key from\n  the root schema and apply the retrieved validators to the data.","ref":"JSV.Key.html"},{"type":"type","title":"JSV.Key.anchor/0","doc":"","ref":"JSV.Key.html#t:anchor/0"},{"type":"type","title":"JSV.Key.dynamic_anchor/0","doc":"","ref":"JSV.Key.html#t:dynamic_anchor/0"},{"type":"function","title":"JSV.Key.for_anchor/2","doc":"Returns an anchor type key.","ref":"JSV.Key.html#for_anchor/2"},{"type":"function","title":"JSV.Key.for_dynamic_anchor/2","doc":"Returns a dynamic anchor type key.","ref":"JSV.Key.html#for_dynamic_anchor/2"},{"type":"function","title":"JSV.Key.for_pointer/2","doc":"Returns a pointer type key.","ref":"JSV.Key.html#for_pointer/2"},{"type":"function","title":"JSV.Key.namespace_of/1","doc":"Returns the namespace of the key.","ref":"JSV.Key.html#namespace_of/1"},{"type":"type","title":"JSV.Key.ns/0","doc":"","ref":"JSV.Key.html#t:ns/0"},{"type":"function","title":"JSV.Key.of/1","doc":"Creates a new key from an external or local reference.","ref":"JSV.Key.html#of/1"},{"type":"type","title":"JSV.Key.pointer/0","doc":"","ref":"JSV.Key.html#t:pointer/0"},{"type":"type","title":"JSV.Key.t/0","doc":"","ref":"JSV.Key.html#t:t/0"},{"type":"function","title":"JSV.Key.to_iodata/1","doc":"Returns a string representation of the key, in a URL/JSON pointer format,\nas chardata.","ref":"JSV.Key.html#to_iodata/1"},{"type":"module","title":"JSV.RNS","doc":"A namespace for a schema ID or reference.\n\nIn the JSV library, each schema or subschema belongs to a namespace.\nSub-schemas with an `$id` property define a new namespace.\n\nA namespace can be `:root` if there is no URI or `$id` to identify a schema or\nan URI without a query string or fragment.","ref":"JSV.RNS.html"},{"type":"function","title":"JSV.RNS.derive/2","doc":"Returns a new string namespace by appending a relative path to a base\nnamespace. If the relative is absolute or `:root`, returns the relative.","ref":"JSV.RNS.html#derive/2"},{"type":"function","title":"JSV.RNS.parse/1","doc":"Parses the given URL or URN and returns an internal representation of its\nnamespace.\n\nAlso accepts `:root` for root schemas without `$id`.","ref":"JSV.RNS.html#parse/1"},{"type":"type","title":"JSV.RNS.t/0","doc":"","ref":"JSV.RNS.html#t:t/0"},{"type":"function","title":"JSV.RNS.to_ns/1","doc":"Returns the string value of the namespace, or `:root`.","ref":"JSV.RNS.html#to_ns/1"},{"type":"module","title":"JSV.Ref","doc":"Representation of a JSON Schema reference (`$ref` or `$dynamicRef`).","ref":"JSV.Ref.html"},{"type":"function","title":"JSV.Ref.escape_json_pointer/1","doc":"Encodes the given string as a JSON representation of a JSON pointer, that is\nwith `~` as `~0` and `/` as `~1`.","ref":"JSV.Ref.html#escape_json_pointer/1"},{"type":"type","title":"JSV.Ref.ns/0","doc":"","ref":"JSV.Ref.html#t:ns/0"},{"type":"function","title":"JSV.Ref.parse/2","doc":"Creates a new reference from an URL, relative to the given namespace.\n\nIf the URL is absolute and its namespace is different from the given\nnamespace, returns an absolute URL.","ref":"JSV.Ref.html#parse/2"},{"type":"function","title":"JSV.Ref.parse!/2","doc":"Raising version of the `parse/2` function.","ref":"JSV.Ref.html#parse!/2"},{"type":"function","title":"JSV.Ref.parse_dynamic/2","doc":"Like `parse/2` but flags the reference as dynamic.","ref":"JSV.Ref.html#parse_dynamic/2"},{"type":"function","title":"JSV.Ref.pointer/2","doc":"Creates a new pointer reference from a list of path segments.\n\nThe segments can be strings or integers, representing the path components\nof a JSON pointer.","ref":"JSV.Ref.html#pointer/2"},{"type":"function","title":"Examples - JSV.Ref.pointer/2","doc":"iex> JSV.Ref.pointer([\"properties\", \"name\"], :root)\n    {:ok, %JSV.Ref{ns: :root, kind: :pointer, arg: [\"properties\", \"name\"], dynamic?: false}}\n\n    iex> JSV.Ref.pointer([\"items\", 0], :root)\n    {:ok, %JSV.Ref{ns: :root, kind: :pointer, arg: [\"items\", 0], dynamic?: false}}","ref":"JSV.Ref.html#pointer/2-examples"},{"type":"function","title":"JSV.Ref.pointer!/2","doc":"Creates a new pointer reference from a list of path segments.\n\nRaising version of the `pointer/2` function.","ref":"JSV.Ref.html#pointer!/2"},{"type":"type","title":"JSV.Ref.t/0","doc":"","ref":"JSV.Ref.html#t:t/0"},{"type":"module","title":"JSV.Resolver.Resolved","doc":"Metadata gathered from a remote schema or a sub-schema.","ref":"JSV.Resolver.Resolved.html"},{"type":"type","title":"JSV.Resolver.Resolved.t/0","doc":"","ref":"JSV.Resolver.Resolved.html#t:t/0"},{"type":"module","title":"JSV.Subschema","doc":"Build result for a raw map schema.","ref":"JSV.Subschema.html"},{"type":"type","title":"JSV.Subschema.t/0","doc":"","ref":"JSV.Subschema.html#t:t/0"},{"type":"type","title":"JSV.Subschema.validators/0","doc":"","ref":"JSV.Subschema.html#t:validators/0"},{"type":"module","title":"JSV.Validator","doc":"This is the home of the recursive validation logic.\n\nThe validator is called on the root schema, and may be called by vocabulary\nimplementations to validate sub parts of the data built withing each\nvocabulary module.","ref":"JSV.Validator.html"},{"type":"type","title":"JSV.Validator.context/0","doc":"","ref":"JSV.Validator.html#t:context/0"},{"type":"function","title":"JSV.Validator.context/3","doc":"","ref":"JSV.Validator.html#context/3"},{"type":"function","title":"JSV.Validator.error?/1","doc":"Returns whether the given context contains errors.","ref":"JSV.Validator.html#error?/1"},{"type":"type","title":"JSV.Validator.eval_sub_path/0","doc":"","ref":"JSV.Validator.html#t:eval_sub_path/0"},{"type":"function","title":"JSV.Validator.flat_errors/1","doc":"","ref":"JSV.Validator.html#flat_errors/1"},{"type":"function","title":"JSV.Validator.list_evaluaded/1","doc":"","ref":"JSV.Validator.html#list_evaluaded/1"},{"type":"function","title":"JSV.Validator.merge_tracked/2","doc":"Merges tracking data from the sub context into the main context. This is\nuseful to keep information defined by subschemas for the same data level as\nthe parent schema. Such sub schemas are defined with oneOf/allOf/... or $ref.\n\nTracking data:\n\n* Evaluated paths, to work with unevaluated properties/items\n* Cast functions (defschema,defcast)","ref":"JSV.Validator.html#merge_tracked/2"},{"type":"function","title":"JSV.Validator.reduce/4","doc":"Reduce over an enum with two accumulators, a user one, and the context.\n\n* The callback is called with `item, acc, vctx` for all items in the enum,\n  regardless of previously returned values. Returning and error tuple does not\n  stop the iteration.\n* When returning `{:ok, value, vctx}`, `value` will be the new user\n  accumulator, and the new context is carried on.\n* When returning `{:error, vctx}`, the current accumulator is not changed, but\n  the new returned context with errors is still carried on.\n* Returning an ok tuple after an error tuple on a previous item does not\n  remove the errors from the context struct.\n\nThe final return value is `{:ok, acc, vctx}` if all calls of the callback\nreturned an OK tuple, `{:error, vctx}` otherwise.\n\nThis is useful to call all possible validators for a given piece of data,\ncollecting all possible errors without stopping, but still returning an error\nin the end if some error arose.","ref":"JSV.Validator.html#reduce/4"},{"type":"type","title":"JSV.Validator.result/0","doc":"","ref":"JSV.Validator.html#t:result/0"},{"type":"function","title":"JSV.Validator.return/2","doc":"","ref":"JSV.Validator.html#return/2"},{"type":"function","title":"JSV.Validator.to_error/1","doc":"","ref":"JSV.Validator.html#to_error/1"},{"type":"function","title":"JSV.Validator.validate/3","doc":"Validate the given data with the given validator. The validator is typically a\nsub-part of a `JSV.Root` struct built with `JSV.build/2` such as a\n`JSV.Subschema` struct.","ref":"JSV.Validator.html#validate/3"},{"type":"function","title":"JSV.Validator.validate_as/4","doc":"Validates data with a sub part of the schema, for instance `if`, `then` or\n`else`. Data path will not change in the context.\n\nSee `validate_in/5` to validate sub terms of the data.","ref":"JSV.Validator.html#validate_as/4"},{"type":"function","title":"JSV.Validator.validate_detach/4","doc":"Validate the data with the given validators but separate the current\nevaluation context during the validation.\n\nCurrently evaluated properties or items will not be seen as evaluated during\nthe validation by the given `subschema`.","ref":"JSV.Validator.html#validate_detach/4"},{"type":"function","title":"JSV.Validator.validate_in/5","doc":"Validates a sub term of the data, identified by `key`, which can be a property\nname (a string), or an array index (an integer).\n\nSee `validate_as/4` to validate the same data point with a nested keyword. For\ninstance `if`, `then` or `else`.","ref":"JSV.Validator.html#validate_in/5"},{"type":"function","title":"JSV.Validator.validate_ref/4","doc":"","ref":"JSV.Validator.html#validate_ref/4"},{"type":"type","title":"JSV.Validator.validator/0","doc":"","ref":"JSV.Validator.html#t:validator/0"},{"type":"type","title":"JSV.Validator.validators/0","doc":"","ref":"JSV.Validator.html#t:validators/0"},{"type":"macro","title":"JSV.Validator.with_error/4","doc":"","ref":"JSV.Validator.html#with_error/4"},{"type":"module","title":"JSV.Validator.Error","doc":"Representation of an error encountered during validation.","ref":"JSV.Validator.Error.html"},{"type":"opaque","title":"JSV.Validator.Error.t/0","doc":"","ref":"JSV.Validator.Error.html#t:t/0"},{"type":"module","title":"JSV.Validator.ValidationContext","doc":"Validation context carried along by the `JSV.Validator` and given to all\nvocabulary implementations.\n\nThis struct is used to store errors found during validation, and to hold\ncontextual information such as the current path in the data or in the\nschema.","ref":"JSV.Validator.ValidationContext.html"},{"type":"type","title":"JSV.Validator.ValidationContext.t/0","doc":"","ref":"JSV.Validator.ValidationContext.html#t:t/0"},{"type":"task","title":"mix jsv.gen_test_suite","doc":"","ref":"Mix.Tasks.Jsv.GenTestSuite.html"},{"type":"function","title":"Mix.Tasks.Jsv.GenTestSuite.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.Jsv.GenTestSuite.html#run/1"},{"type":"function","title":"Mix.Tasks.Jsv.GenTestSuite.stream_cases/3","doc":"","ref":"Mix.Tasks.Jsv.GenTestSuite.html#stream_cases/3"},{"type":"function","title":"Mix.Tasks.Jsv.GenTestSuite.suite_dir!/1","doc":"","ref":"Mix.Tasks.Jsv.GenTestSuite.html#suite_dir!/1"},{"type":"task","title":"mix jsv.gen_test_suite.value_dumper","doc":"","ref":"Mix.Tasks.Jsv.GenTestSuite.ValueDumper.html"},{"type":"function","title":"Mix.Tasks.Jsv.GenTestSuite.ValueDumper.render/2","doc":"","ref":"Mix.Tasks.Jsv.GenTestSuite.ValueDumper.html#render/2"},{"type":"function","title":"Mix.Tasks.Jsv.GenTestSuite.ValueDumper.to_ordlist/1","doc":"","ref":"Mix.Tasks.Jsv.GenTestSuite.ValueDumper.html#to_ordlist/1"},{"type":"function","title":"Mix.Tasks.Jsv.GenTestSuite.ValueDumper.wrap/2","doc":"","ref":"Mix.Tasks.Jsv.GenTestSuite.ValueDumper.html#wrap/2"},{"type":"task","title":"mix jsv.update_jsts_ref","doc":"","ref":"Mix.Tasks.Jsv.UpdateJstsRef.html"},{"type":"function","title":"Mix.Tasks.Jsv.UpdateJstsRef.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.Jsv.UpdateJstsRef.html#run/1"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\n## [0.10.0] - 2025-07-10","ref":"changelog.html"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Define and expect schema modules to export json_schema/0 instead of schema/0\n- Allow to call defschema with a list of properties\n- Added the defschema/3 macro to define schemas as submodules","ref":"changelog.html#features"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Ensure defschema with keyword syntax supports module-based properties\n\n## [0.9.0] - 2025-07-05","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Provide a schema representing normalized validation errors\n- Deprecated the schema composition API in favor of presets","ref":"changelog.html#features-1"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Emit a build error with empty oneOf/allOf/anyOf\n- Reset errors when using a detached validator\n- Ensure casts are applied after all validations\n- Revert default normalized error to atoms","ref":"changelog.html#bug-fixes-1"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Define titles for normal validation error schemas\n\n## [0.8.1] - 2025-06-29","ref":"changelog.html#miscellaneous-tasks"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Export the locals_without_parens formatter opts for public macros\n\n## [0.8.0] - 2025-06-23","ref":"changelog.html#miscellaneous-tasks-1"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Declare formatting support from main JSON codec\n- Added the JSV.validate! bang functions\n- Added explicit error when a sub schema is not buildable\n- Export JSV.resolver_chain/1 for integration in 3rd parties\n- [**breaking**] Defschema does not automatically define $id anymore\n- Added string_to_number and string_to_boolean casters\n- Return sub errors when oneOf has no matches\n- Order sub-errors by ascending item index in array validation\n- Added ability to build only a nested schema or multiple schemas\n- Expose the map extensions helpers\n- Added the prewalk traverse utility for schema normalization\n- [**breaking**] Error normalizer will now sort error by instanceLocation\n- [**breaking**] Changed caster tag of defschema to 0\n- Allow custom formats to validate other types than strings\n- Provide a function to create reference from a list of path segments","ref":"changelog.html#features-2"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Ensure keys are json-pointer encoded in instanceLoction in errors\n- Return meaningful error for unknow keys in :required in defschema\n- Fixed typespec on JSV.build_key!\n- Fixed typespec and argument name in Builder.build!","ref":"changelog.html#bug-fixes-2"},{"type":"extras","title":"🚜 Refactor - Changelog","doc":"- Renamed Schema.override/2 to Schema.merge/2\n- Defined different typespecs for normal schema and native schema\n- Build error will now be raised with a proper stacktrace\n- Removed useless accumulation of atoms when normalizing schemas\n- [**breaking**] Changed order of arguments for Normalizer.normalize/3\n- Renamed build_root to to_root as it is not building validators","ref":"changelog.html#refactor"},{"type":"extras","title":"📚 Documentation - Changelog","doc":"- Rework Decimal support limitations","ref":"changelog.html#documentation"},{"type":"extras","title":"🧪 Testing - Changelog","doc":"- Verify that unknown formats are ignored when formats assertion is disabled","ref":"changelog.html#testing"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Clarify defschema error when no properties are given\n- Fix warning when Poison.EncodeError is not defined\n- Updated JSON Schema Test Suite\n- Renamed keycast module attribute to jsv_keycast in defschema\n- Provide correct line/column in debanged functions\n- Allow to customize Inspect for Builder and Resolver\n- Fix Elixir 1.19 warnings\n\n## [0.7.2] - 2025-05-08","ref":"changelog.html#miscellaneous-tasks-2"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Added the non_empty_string schema helper\n- Atom enums will use string_to_atom to support compile-time builds","ref":"changelog.html#features-3"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Updated JSON Schema Test Suite\n- Enhanced JSTS updater\n- Fixed warning on code when Decimal is missing\n\n## [0.7.1] - 2025-04-27","ref":"changelog.html#miscellaneous-tasks-3"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Fixed hex package definition\n\n## [0.7.0] - 2025-04-27","ref":"changelog.html#bug-fixes-3"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Mail_address dependency is no longer used\n- Validation support for Decimal","ref":"changelog.html#features-4"},{"type":"extras","title":"📚 Documentation - Changelog","doc":"- Updated doc examples with generated code","ref":"changelog.html#documentation-1"},{"type":"extras","title":"🧪 Testing - Changelog","doc":"- Enable tests for the 'uuid' format\n- Enable tests for the 'hostname' format\n- Enable tests for all uri/iri/pointer formats","ref":"changelog.html#testing-1"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Changed JSON schema test suite updater\n\n## [0.6.3] - 2025-04-13","ref":"changelog.html#miscellaneous-tasks-4"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Fix missing file in hex package breaking installs\n\n## [0.6.2] - 2025-04-13","ref":"changelog.html#miscellaneous-tasks-5"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Added Jason/Poison/JSON encoder implementations for JSV.NValidationError\n\n## [0.6.1] - 2025-04-13","ref":"changelog.html#features-5"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Use mix_version for release process\n\n## [0.6.0] - 2025-04-13","ref":"changelog.html#miscellaneous-tasks-6"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Resolvers do not need to normalize schemas anymore\n- Added support to override existing vocabularies\n- Schema definition helpers do not enforce a Schema struct anymore\n- Provide a generic JSON normalizer for data and schemas\n- Allow resolvers to mark schemas as normalized\n- [**breaking**] Use jsv-cast keyword in schemas for struct and cast functions","ref":"changelog.html#features-6"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Removed conversion to string in codec format_to_iodata","ref":"changelog.html#bug-fixes-4"},{"type":"extras","title":"📚 Documentation - Changelog","doc":"- Fix documentation grammar and typos\n- Organize docs sidebar in categories","ref":"changelog.html#documentation-2"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Update Elixir Github workflow (#17)\n- Use absolute path for JSTS ref file\n\n## [0.5.1] - 2025-03-28","ref":"changelog.html#miscellaneous-tasks-7"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Fixed compilation with Mix.install","ref":"changelog.html#bug-fixes-5"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Release v0.5.1\n\n## [0.5.0] - 2025-03-25","ref":"changelog.html#miscellaneous-tasks-8"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Added JSV.Resolver.Local to resolve disk stored schemas\n- Special error format for additionalProperties:false\n- Provide correct schemaLocation in all errors\n- Added defschema_for to use different modules for schema and struct\n- Provide ordered JSON encoding with native JSON modules","ref":"changelog.html#features-7"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Check presence of JSON module in CI","ref":"changelog.html#bug-fixes-6"},{"type":"extras","title":"🧪 Testing - Changelog","doc":"- Make JSON codecs easier to test\n- Fixed assertions for JSON codec on old OTP versions","ref":"changelog.html#testing-2"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Refactored schema normalization\n- Removed unused alias\n- Use readmix to generate formats docs\n\n## [0.4.0] - 2025-02-08","ref":"changelog.html#miscellaneous-tasks-9"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Support module-based schemas with structs\n\n## [0.3.0] - 2025-01-08","ref":"changelog.html#features-8"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Added a default resolver using static schemas","ref":"changelog.html#features-9"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Upgrade abnf_parsec to correctly parse IRIs and IRI references\n\n## [0.2.0] - 2025-01-03","ref":"changelog.html#bug-fixes-7"},{"type":"extras","title":"📚 Documentation - Changelog","doc":"- Document atom conversion\n- Document functions with doc and spec\n\n## [0.1.0] - 2025-01-01\n\n<!-- generated by git-cliff -->","ref":"changelog.html#documentation-3"},{"type":"extras","title":"Defining Schemas","doc":"# Defining Schemas\n\nThis guide explains the different possible values to use as a schema with JSV.","ref":"defining-schemas.html"},{"type":"extras","title":"Schema sources - Defining Schemas","doc":"Schemas given as input to JSV can come from two sources:\n\n* Schemas given directly to the `JSV.build/2` function. These are expected to be\n  maps or booleans. JSV will not parse JSON strings.\n* Schemas returned by resolvers. These should also be maps (but not booleans).\n  The built-in resolvers will handle JSON deserialization automatically.\n\nJSV is designed to work with raw schemas. Any map or boolean is a valid schema.\nFor instance, it is possible to directly use a schema from a file:\n\n<!-- rdmx :section name:schema_from_file format:true -->\n```elixir\nroot =\n  \"my-schema.json\"\n  |> File.read!()\n  |> JSON.decode!()\n  |> JSV.build()\n```\n<!-- rdmx /:section -->","ref":"defining-schemas.html#schema-sources"},{"type":"extras","title":"Schema formats - Defining Schemas","doc":"Schemas can be either booleans or maps. The `true` value is equivalent to an\nempty JSON Schema `{}`, while `false` is a schema that will invalidate any\nvalue. It is most often used as a sub-schema for `additionalProperties`.\n\nMaps can define keys and values as binaries or atoms. The following schemas are\nequivalent:\n\n<!-- rdmx :section name:equivalent_schemas format:true -->\n```elixir\n%{type: :boolean}\n%{type: \"boolean\"}\n%{\"type\" => \"boolean\"}\n# You will rarely find this one in the wild!\n%{\"type\" => :boolean}\n```\n<!-- rdmx /:section -->\n\nThis is because JSV will normalize the schemas before building a \"root\", the\nbase data structure for data validation.\n\nMixing keys is not recommended. In the following example, JSV will build a\nschema that will successfully validate integers with a minimum of zero. However,\nthe choice for the maximum value is not made by JSV.\n\n<!-- rdmx :section name:mixed_keys format:true -->\n```elixir\n%{:type => :integer, \"minimum\" => 0, \"maximum\" => 10, :maximum => 20}\n```\n<!-- rdmx /:section -->","ref":"defining-schemas.html#schema-formats"},{"type":"extras","title":"Struct schemas - Defining Schemas","doc":"Schemas can be used to define structs with the `JSV.defschema/1` macro.\n\nFor instance, with this module definition schema:\n\n<!-- rdmx :section name:struct_schema format:true -->\n```elixir\ndefmodule MyApp.UserSchema do\n  use JSV.Schema\n\n  defschema %{\n    type: :object,\n    properties: %{\n      name: %{type: :string, default: \"\"},\n      age: %{type: :integer, default: 0}\n    }\n  }\nend\n```\n<!-- rdmx /:section -->\n\nA struct will be defined with the appropriate default values:\n\n```elixir\niex> %MyApp.UserSchema{}\n%MyApp.UserSchema{name: \"\", age: 0}\n```\n\nThe module can be used as a schema to build a validator root and cast data to\nthe corresponding struct:\n\n```elixir\niex> {:ok, root} = JSV.build(MyApp.UserSchema)\niex> data = %{\"name\" => \"Alice\"}\niex> JSV.validate(data, root)\n{:ok, %MyApp.UserSchema{name: \"Alice\", age: 0}}\n```\n\nCasting to a struct can be disabled by passing `cast: false` into the\noptions of `JSV.validate/3`.\n\n```elixir\niex> {:ok, root} = JSV.build(MyApp.UserSchema)\niex> data = %{\"name\" => \"Alice\", \"extra\" => \"hello!\"}\niex> JSV.validate(data, root, cast: false)\n{:ok, %{\"name\" => \"Alice\", \"extra\" => \"hello!\"}}\n```\n\nThe module can also be used in other schemas:\n\n<!-- rdmx :section name:nested_schema format:true -->\n```elixir\n%{\n  type: :object,\n  properties: %{\n    name: %{type: :string},\n    owner: MyApp.UserSchema\n  }\n}\n```\n<!-- rdmx /:section -->\n\nAn alternative syntax can be used, by passing only the properties schemas as a\nlist.\n\n<!-- rdmx :section name:alternative_syntax format:true -->\n```elixir\ndefmodule MyApp.UserSchema do\n  use JSV.Schema\n\n  defschema name: %{type: :string, default: \"\"},\n            age: %{type: :integer, default: 0}\nend\n```\n<!-- rdmx /:section -->\n\nIn that case, properties that do not have a default value are automatically\nrequired, and the `type` of the schema is automatically set to `object`. The\n`title` of the schema is set as the last segment of the module name.\n\nStruct defining schemas are a special case of the generic cast mechanism built\nin JSV. Make sure to check that guide out as well.","ref":"defining-schemas.html#struct-schemas"},{"type":"extras","title":"Defining multiple schemas with defschema/3 - Defining Schemas","doc":"The `JSV.defschema/3` macro allows you to define a new module for a schema. It\ncan be used inside an enclosing \"group\" module, which is useful for organizing\nrelated schemas together:\n\n<!-- rdmx :section name:multiple_schemas format:true -->\n```elixir\ndefmodule MyApp.Schemas do\n  use JSV.Schema\n\n  defschema User,\n    name: string(),\n    email: string(),\n    age: integer(default: 18)\n\n  defschema Address,\n            \"Physical address information\",\n            street: string(),\n            city: string(),\n            country: string(default: \"US\")\n\n  defschema Company,\n            \"Company information with nested schemas\",\n            name: string(),\n            address: Address,\n            employees: array_of(User)\nend\n```\n<!-- rdmx /:section -->\n\nThis creates three separate modules: `MyApp.Schemas.User`,\n`MyApp.Schemas.Address`, and `MyApp.Schemas.Company`, each with their own struct\nand JSON schema.\n\nYou can use these schemas independently:\n\n<!-- rdmx :section name:independent_usage format:true -->\n```elixir\nuser = %MyApp.Schemas.User{name: \"Alice\", email: \"alice@example.com\"}\naddress = %MyApp.Schemas.Address{street: \"123 Main St\", city: \"Boston\"}\n\n{:ok, user_root} = JSV.build(MyApp.Schemas.User)\n{:ok, company_root} = JSV.build(MyApp.Schemas.Company)\n```\n<!-- rdmx /:section -->","ref":"defining-schemas.html#defining-multiple-schemas-with-defschema-3"},{"type":"extras","title":"Using full schema maps - Defining Schemas","doc":"You can also use `defschema/3` with complete schema maps instead of property lists:\n\n<!-- rdmx :section name:full_schema_maps format:true -->\n```elixir\ndefmodule MyApp.Schemas do\n  use JSV.Schema\n\n  defschema ApiResponse,\n            \"Standard API response format\",\n            %{\n              type: :object,\n              properties: %{\n                success: %{type: :boolean},\n                data: %{type: :object},\n                errors: %{type: :array, items: %{type: :string}}\n              },\n              required: [:success],\n              additionalProperties: false\n            }\nend\n```\n<!-- rdmx /:section -->\n\nWhen using full schema maps, the title and description from the macro parameters\nare **not** automatically applied to the schema, the map is used as-is. Only the\ndescription parameter is used for the module's documentation.","ref":"defining-schemas.html#using-full-schema-maps"},{"type":"extras","title":"Self-referencing schemas - Defining Schemas","doc":"Schemas can reference themselves using `__MODULE__`:\n\n<!-- rdmx :section name:self_reference format:true -->\n```elixir\ndefmodule MyApp.Schemas do\n  use JSV.Schema\n\n  defschema Category,\n            \"Hierarchical category structure\",\n            name: string(),\n            parent: optional(__MODULE__)\nend\n```\n<!-- rdmx /:section -->\n\nThis creates a `MyApp.Schemas.Category` module that can have a parent of the\nsame type, allowing for hierarchical data structures.","ref":"defining-schemas.html#self-referencing-schemas"},{"type":"extras","title":"Custom Cast Functions","doc":"# Custom Cast Functions\n\nJSV provides a mechanism to declare a cast function into a schema, to be called\nonce the validation is successful. This is the same mechanism used to cast\nstruct-schemas to Elixir structs.\n\nThis guide describes how to use a custom function in your schemas.","ref":"cast-functions.html"},{"type":"extras","title":"JSV's cast system - Custom Cast Functions","doc":"JSV stores the cast information in the JSON schemas. There is no central\nregistry of cast functions, so any library you use can define its own JSV casts\nwithout needing you to copy their mapping, registry or whatever in your\nconfiguration.\n\nThat information is stored under the `jsv-cast` custom schema keyword as an\narray containing a module name (as string) and a custom \"tag\".\n\n```json\n{\n  \"description\": \"an existing Elixir atom\",\n  \"type\": \"string\",\n  \"jsv-cast\": [\"Elixir.MyApp.Schemas.Cast\", \"existing_atom\"]\n}\n```\n\nThis solution has multiple advantages:\n\n* No configuration.\n* Schemas remain fully declarative. The information about casting is collocated\n  with other validation keywords such as `type`, `format`, `properties`, _etc_.\n* Schemas are portable as JSV does not need additional configuration to know\n  what code to call. Although, we use a module name so the module needs to be\n  available in the Elixir runtime when data is validated.\n* Schema reviewers can know that a schema uses a cast without needing to look\n  elsewhere.\n* Cast functions can be referenced into multiple schemas, they are not tied to a\n  particular struct or schema-map defined in one place. You can also define them\n  in generic schemas referenced with `$ref` or `$dynamicRef`.\n\nThere are some drawbacks as well:\n\n* The `jsv-cast` information needs to be JSON-serializable, so modules are\n  referenced as strings, and custom tags to identify the cast function can only\n  be simple data. Currently JSV only accepts strings or integers.\n* Module names are leaked into the schemas. If this is not acceptable, you can\n  declare a generic \"Cast\" module in your application and dispatch manually from\n  there. Sometimes just cleaning the schemas before making them public is enough\n  too.\n* Refactoring can be harder. In general, you will not write the content of\n  `jsv-cast` by hand but rather use our helper functions. Refactoring will be\n  the same as with regular code.\n* Indirection for the cast functions is required. See the security concerns\n  below.","ref":"cast-functions.html#jsv-s-cast-system"},{"type":"extras","title":"Security concerns - Custom Cast Functions","doc":"In the previous example, `\"existing_atom\"` is a \"tag\", and not a function name\nthat JSV would call blindly. Otherwise, if your app is processing third-party\nschemas, a `[\"Elixir.System\", \"stop\"]` or worse would be very bad.\n\nFor that reason, cast functions need to be enabled by developers by defining the\n`__jsv__/2` function.\n\nThis is an internal callback, not documented by a behaviour, but security is\nimportant and it is worth explaining the mechanism here. The `__jsv__/2`\nfunction is rather defined by using the `JSV.defcast/1` macro that is documented.\n\nWhen evaluating `[\"Elixir.System\", \"stop\"]`, JSV will indeed call the `System`\nmodule. It will call `System.__jsv__(\"stop\", data)` with the data under\nvalidation. This function does not exist and JSV will catch that error.\n\nThe only way for that function to exist is if you define it in your own code.\nWhile you could compile a custom Elixir version with a `__jsv__/2` function in\nthe `System` module, there are only so many reasons to do that.\n\nBut that applies to your modules as well. Only you can define the `__jsv__/2`\nfunction in your modules.\n\nWhile this requires a few extra lines of code, we think it's a simple-enough\nsolution to prevent undesirable remote code execution.","ref":"cast-functions.html#security-concerns"},{"type":"extras","title":"Defining cast functions - Custom Cast Functions","doc":"Cast functions are functions that return a generic result tuple:\n- `{:ok, transformed_data}` for successful transformations.\n- `{:error, reason}` when the transformation fails.\n\nAs described in the security section above, JSV needs a tag to identify the\nfunction to call with the data. JSV supports strings and integers as tags.\n\nTo define such functions, the `JSV.defcast/1`, `JSV.defcast/2` or\n`JSV.defcast/3` macros should be used.","ref":"cast-functions.html#defining-cast-functions"},{"type":"extras","title":"Basic usage of defcast - Custom Cast Functions","doc":"The following module expects a string and returns the value in upper case:\n\n```elixir\ndefmodule MyApp.Schemas.Cast do\n  import JSV\n\n  defcast to_uppercase(data) do\n    {:ok, String.upcase(data)}\n  end\nend\n```\n\nThis will define the `to_uppercase/1` function that will evaluate the body as\nany regular function:\n\n```elixir\nMyApp.Schemas.Cast.to_uppercase(\"hello\")\n# => {:ok, \"HELLO\"}\n```\n\nIt will also define the `to_uppercase/0` function that returns the cast\ninformation to include in a schema. As you can see, the default tag of a cast is\nthe function name, as string.\n\n```elixir\nMyApp.Schemas.Cast.to_uppercase()\n# => [\"Elixir.MyApp.Schemas.Cast\", \"to_uppercase\"]\n```\n\nAnd finally, it will define the appropriate `__jsv__/2` function to be used in a\nschema:\n\n```elixir\nschema = JSV.Schema.string() |> JSV.Schema.with_cast(MyApp.Schemas.Cast.to_uppercase())\n# => %JSV.Schema{\n#      type: :string,\n#      \"jsv-cast\": [\"Elixir.MyApp.Schemas.Cast\", \"to_uppercase\"]\n#    }\n\nroot = JSV.build!(schema)\nJSV.validate(\"hello\", root)\n# => {:ok, \"HELLO\"}\n```","ref":"cast-functions.html#basic-usage-of-defcast"},{"type":"extras","title":"Using a custom tag - Custom Cast Functions","doc":"Custom tags can be given as the first argument of `defcast`:\n\n```elixir\n# Using a string tag\ndefcast \"my_custom_tag\", to_uppercase(data) do\n  {:ok, String.upcase(data)}\nend\n\n# Using an integer tag\ndefcast ?u, to_uppercase(data) do\n  {:ok, data}\nend\n```","ref":"cast-functions.html#using-a-custom-tag"},{"type":"extras","title":"Exception handling - Custom Cast Functions","doc":"The `rescue`, `catch` and `after` blocks are supported:\n\n```elixir\ndefcast safe_to_atom(data) do\n  {:ok, String.to_existing_atom(data)}\nrescue\n  ArgumentError -> {:error, :unknown_atom}\nend\n```","ref":"cast-functions.html#exception-handling"},{"type":"extras","title":"Referring to Existing Functions - Custom Cast Functions","doc":"Guards with the `when` keyword are not supported at the moment. But it is\npossible to call a local function instead of defining it directly.\n\nUnfortunately, this does not export the 0-arity function to quickly define the\ncast in schemas.\n\n```elixir\n# Pass the local function name as a single argument.\ndefcast :to_upper\n\n# Custom tags are supported too\ndefcast \"custom_tag\", :to_upper\ndefcast ?u, :to_upper\n\n# The function needs to be\n# defined in the module as `def` or `defp`.\ndef to_upper(data) when is_binary(data), do: {:ok, String.upcase(data)}\ndef to_upper(data), do: {:error, :expected_string}\n\n# Helper to reference the cast in schemas. This function does not need to\n# exist, it is always possible to add this directly in a schema.\ndef to_upper do\n  [to_string(__MODULE__), \"to_upper\"]\nend\n```","ref":"cast-functions.html#referring-to-existing-functions"},{"type":"extras","title":"Error Normalization - Custom Cast Functions","doc":"To return custom errors from your functions, you can optionally define the\n`format_error/3` function that will receive the tag, the `reason` and the\nvalidated data.\n\nThis will be called when JSV errors are normalized to be JSON-encodable.\n\n<!-- rdmx :section name:example_error format:true -->\n```elixir\ndefmodule MyApp.Schemas.Cast do\n  import JSV\n\n  defcast safe_to_atom(data) do\n    {:ok, String.to_existing_atom(data)}\n  rescue\n    ArgumentError -> {:error, :unknown_atom}\n  end\n\n  def format_error(\"safe_to_atom\", :unknown_atom, data) do\n    \"could not cast to existing atom: #{inspect(data)}\"\n  end\nend\n\nschema = JSV.Schema.Helpers.string() |> JSV.Schema.with_cast(MyApp.Schemas.Cast.safe_to_atom())\n\nroot = JSV.build!(schema)\n{:error, err} = JSV.validate(\"some string\", root)\nJSV.normalize_error(err)\n```\n<!-- rdmx /:section -->\n\nThe code above gies the following normalized error:\n\n<!-- rdmx :eval section:example_error  -->\n```elixir\n%{\n  details: [\n    %{\n      errors: [\n        %{\n          kind: :cast,\n          message: \"could not cast to existing atom: \\\"some string\\\"\"\n        }\n      ],\n      evaluationPath: \"#\",\n      instanceLocation: \"#\",\n      schemaLocation: \"#\",\n      valid: false\n    }\n  ],\n  valid: false\n}\n```\n<!-- rdmx /:eval -->","ref":"cast-functions.html#error-normalization"},{"type":"extras","title":"Building Schemas","doc":"# Building Schemas\n\nTo validate data with schemas, `JSV` turns the schemas into a custom data\nstructure made specifically for validation.\n\nIt does not validate data with raw schemas directly. That would be too slow and\nwould not work properly with advanced features of Draft 2020-12, such as\n`$dynamicRef` and `$dynamicAnchor`.\n\nInstead, raw schemas are processed by a set of different \"vocabulary\" modules\nthat are each specialized in some part of the validation. The result of this\nprocessing is then collected as a `JSV.Root` struct.\n\nThis guide covers the configuration and customization of this process to better\nserve your needs.","ref":"build-basics.html"},{"type":"extras","title":"The build functions - Building Schemas","doc":"The main build function is `JSV.build/2`. It accepts a raw schema and a set of\noptions and returns the root.\n\nThere are variations around that function, which is very common in Elixir:\n`JSV.build/1` with default options, `JSV.build!/1`, and `JSV.build!/2` with or\nwithout default options that raise errors instead of returning an error tuple.","ref":"build-basics.html#the-build-functions"},{"type":"extras","title":"Custom build modules - Building Schemas","doc":"The build functions do not use macros or process-based techniques. We\nencourage you to wrap them and define your options in a single place:\n\n```elixir\ndefmodule MyApp.SchemaBuilder do\n\n  def build(raw_schema) do\n    JSV.build(raw_schema, build_opts())\n  end\n\n  def build!(raw_schema) do\n    JSV.build!(raw_schema, build_opts())\n  end\n\n  defp build_opts do\n    [resolver: MyApp.CustomSchemaResolver, formats: true]\n  end\nend\n```","ref":"build-basics.html#custom-build-modules"},{"type":"extras","title":"Compile-time builds - Building Schemas","doc":"Validation roots can be built at runtime, but it is recommended to build them\nduring compilation, if possible, to avoid repeating the build step unnecessarily.\n\nBuilding at runtime should be done when the JSON schema is not available during\ncompilation.\n\nFor instance, if we have this function that should validate external data:\n\n```elixir\n# DO NOT DO THIS\n\ndefp order_schema do\n  \"priv/schemas/order.schema.json\"\n    |> File.read!()\n    |> JSON.decode!()\n    |> JSV.build!()\nend\n\ndef validate_order(order) do\n  case JSV.validate(order, order_schema()) do\n    {:ok, _} -> OrderHandler.handle_order(order)\n    {:error, _} = err -> err\n  end\nend\n```\n\nThe schema will be built each time the function is called. Building a schema is\nactually pretty fast, but it is a waste of resources nevertheless. In this\nexample, it is obvious that you would not want to read from a file in every call\nto `validate_order`. But the schema fetching will generally be wrapped in a\ncustom function or a build module as suggested above.\n\nMake sure those builds are called at compile-time:\n\n```elixir\n# Do this instead\n\n@order_schema \"priv/schemas/order.schema.json\"\n              |> File.read!()\n              |> JSON.decode!()\n              |> JSV.build!()\n\ndefp order_schema, do: @order_schema\n\ndef validate_order(order) do\n  case JSV.validate(order, order_schema()) do\n    {:ok, _} -> OrderHandler.handle_order(order)\n    {:error, _} = err -> err\n  end\nend\n```","ref":"build-basics.html#compile-time-builds"},{"type":"extras","title":"Enable format validation - Building Schemas","doc":">","ref":"build-basics.html#enable-format-validation"},{"type":"extras","title":"No format validation by default - Building Schemas","doc":"> By default, the `https://json-schema.org/draft/2020-12/schema` meta schema\n> **does not perform format validation**. This is very counterintuitive, but it\n> basically means that the following code is correct:\n\n```elixir\nroot = JSV.build!(%{type: :string, format: :date})\n{:ok, \"not a date\"} = JSV.validate(\"not a date\", root)\n```\n\nThe `format` schema keyword is totally ignored. This is bad, but it is the spec!\nTo always enable format validation when building a root schema, provide the\n`formats: true` option to `JSV.build/2`:\n\n```elixir\nJSV.build(raw_schema, formats: true)\n```\n\nFormat validation is determined at build time. There is no way to change whether\nit is performed once the root schema is built.\n\nThis is another reason to wrap `JSV.build/2` with a custom builder module, so\nyou don't forget to enable those.","ref":"build-basics.html#no-format-validation-by-default"},{"type":"extras","title":"Enable format validation using vocabularies - Building Schemas","doc":"You can also enable format validation by using the JSON Schema specification\nsemantics, though **it is far simpler and less error-prone to use the `:formats`\noption**.\n\nFor format validation to be enabled, a schema should declare the\n`https://json-schema.org/draft/2020-12/vocab/format-assertion` vocabulary\ninstead of the `https://json-schema.org/draft/2020-12/vocab/format-annotation`\none that is included by default in the\n`https://json-schema.org/draft/2020-12/schema` meta schema.","ref":"build-basics.html#enable-format-validation-using-vocabularies"},{"type":"extras","title":"1. Use a new meta schema with format-assertion - Building Schemas","doc":"```json\n{\n    \"$id\": \"custom://with-formats-on/\",\n    \"$schema\": \"https://json-schema.org/draft/2020-12/schema\",\n    \"$vocabulary\": {\n        \"https://json-schema.org/draft/2020-12/vocab/core\": true,\n        \"https://json-schema.org/draft/2020-12/vocab/format-assertion\": true\n    },\n    \"$dynamicAnchor\": \"meta\",\n    \"allOf\": [\n        { \"$ref\": \"https://json-schema.org/draft/2020-12/meta/core\" },\n        { \"$ref\": \"https://json-schema.org/draft/2020-12/meta/format-assertion\" }\n    ]\n}\n```\n\nThis example is taken from the [JSON Schema Test\nSuite](https://github.com/json-schema-org/JSON-Schema-Test-Suite) codebase and\ndoes not include all the vocabularies, only the assertion for the formats and\nthe core vocabulary. It will not validate anything other than formats.","ref":"build-basics.html#1-use-a-new-meta-schema-with-format-assertion"},{"type":"extras","title":"2. Declare a schema using that meta schema to perform validation. - Building Schemas","doc":"You will need a [custom resolver](guides/build/resolvers.md) to resolve the\ngiven URL for the new `$schema` property.\n\n```elixir\nschema =\n  JSON.decode!(\"\"\"\n  {\n    \"$schema\": \"custom://with-formats-on/\",\n    \"type\": \"string\",\n    \"format\": \"date\"\n  }\n  \"\"\")\n\nroot = JSV.build!(schema, resolver: ...)\n```","ref":"build-basics.html#2-declare-a-schema-using-that-meta-schema-to-perform-validation"},{"type":"extras","title":"3. Validate - Building Schemas","doc":"Now it will work as expected. `JSV.validate/2` returns an error tuple without\nneeding the `formats: true`.\n\n```elixir\n{:error, _} = JSV.validate(\"not a date\", root)\n```","ref":"build-basics.html#3-validate"},{"type":"extras","title":"Reverse use-case - Building Schemas","doc":"If one of your schemas is using such a meta schema and you want to _disable_ the\nformats validation, then the following will work:\n\n```elixir\nJSV.build(raw_schema, formats: false)\n```","ref":"build-basics.html#reverse-use-case"},{"type":"extras","title":"Resolvers","doc":"# Resolvers\n\n`JSV` may have to fetch other schemas when building a validation root. This\nhappens with `$schema`, `$ref`, or `$dynamicRef` properties pointing to an\nabsolute [URI](https://en.wikipedia.org/wiki/Uniform_Resource_Identifier).\n\nIn order to fetch those schemas, JSV requires a resolver. Resolvers are\nuser-defined, but JSV provides implementations for common use cases:\n\n* `JSV.Resolver.Embedded` will resolve the most often used meta-schemas such as\n  `https://json-schema.org/draft/2020-12/schema`.\n* `JSV.Resolver.Internal` will resolve struct schemas given as module names as\n  in the example just above.\n* `JSV.Resolver.Httpc` will resolve schemas whose URI are `http` or `https`\n  URLs. It uses the built-in Erlang HTTP client. While not packing many\n  features, it does not enforce an HTTP client dependency in your application.","ref":"resolvers.html"},{"type":"extras","title":"Using resolvers - Resolvers","doc":"The `JSV.Resolver.Embedded` and `JSV.Resolver.Internal` are always enabled and\nthere is no need to declare them when building the root.\n\nOther resolvers such as `JSV.Resolver.Httpc` or custom resolvers (see below)\nneed to be explicitly declared in the `:resolver` option of `JSV.build/2` or\n`JSV.build!/2`:\n\n```elixir\nresolver = {JSV.Resolvers.Httpc, allowed_prefixes: [\"https://example.com/schemas/\"]}\nroot = JSV.build!(schema, resolver: resolver)\n```\n\nMultiple resolvers can be passed as a list:\n\n```elixir\nroot = JSV.build!(schema, resolver: [MyCustomResolver, MyOtherResolver])\n```","ref":"resolvers.html#using-resolvers"},{"type":"extras","title":"Custom resolvers - Resolvers","doc":"Users are encouraged to write their own resolver to support advanced use cases.\n\nCustom resolvers are most often used for:\n\n- Resolving URLs such as `my-company://some-id/` where the implementation knows\n  a directory to retrieve that path from.\n- Resolving `https://` URLs with custom network setups involving authentication,\n  proxies, etc., or to use your HTTP library of choice.\n- Returning hardcoded schemas directly from the codebase.\n- Returning a schema dynamically, for instance depending on the `:prod` or\n  `:test` environment.\n\nTo write a custom resolver, define a module that implements the `JSV.Resolver`\nbehaviour.","ref":"resolvers.html#custom-resolvers"},{"type":"extras","title":"A basic resolver implementation - Resolvers","doc":"```elixir\ndefmodule MyApp.SchemaResolver do\n  @behaviour JSV.Resolver\n\n  @user_schema %{type: :object, properties: %{name: %{type: :string}}}\n  @website_schema %{type: :object, properties: %{url: %{type: :string}}}\n\n  @impl true\n  def resolve(\"myapp:user\", _opts), do: {:ok, @user_schema}\n  def resolve(\"myapp:website\", _opts), do: {:ok, @website_schema}\n  def resolve(_, _opts), do: {:error, :unknown}\nend\n```","ref":"resolvers.html#a-basic-resolver-implementation"},{"type":"extras","title":"Resolving local files - Resolvers","doc":"The `JSV.Resolver.Local` helper can automatically load schemas from files and\ndirectories. Schemas will be resolvable by their `$id` property.\n\n```elixir\ndefmodule MyApp.LocalResolver do\n  use JSV.Resolver.Local, source: [\n    \"priv/api/schemas\",\n    \"priv/message-queue/schemas\",\n    \"priv/special.file.json\"\n  ]\nend\n```\n\nMake sure to check the documentation of `JSV.Resolver.Local` for more\ninformation.","ref":"resolvers.html#resolving-local-files"},{"type":"extras","title":"Returning normalized schemas - Resolvers","doc":"If your resolver returns JSON data that is in normal form like this:\n\n```elixir\n%{\n  \"type\" => \"object\",\n  \"additionalProperties\" => false\n}\n```\n\nThen the resolver implementation can return `{:normal, schema}` instead of\n`{:ok, schema}` to skip the normalization step operated by JSV when building a\nvalidation root.\n\nThe following form is _not_ normal and requires normalization:\n\n```elixir\n%{\n  type: => :object,\n  additionalProperties: => false\n}\n```\n\nSee `JSV.Normalizer.normalize/3` for more details.","ref":"resolvers.html#returning-normalized-schemas"},{"type":"extras","title":"Resolvers form a chain - Resolvers","doc":"As mentioned before, the `JSV.Resolver.Embedded` and `JSV.Resolver.Internal` are\nalways enabled when calling `JSV.build/2`. This means that when calling the\nfollowing code:\n\n```elixir\nroot = JSV.build(schema, resolver: [MyCustomResolver, MyOtherResolver])\n```\n\nThe actual list of used resolvers is\n\n* `MyCustomResolver`\n* `MyOtherResolver`\n* `JSV.Resolver.Embedded`\n* `JSV.Resolver.Internal`\n\nJSV will try each resolver in order until a successful response is returned, and\nfail if _all_ of them return an error.\n\n> #### Don't break the chain {: .warning}\n>\n> Make sure to define a catch-all clause for `resolve/2` in your implementation\n> to return an `{:error, _}` tuple and allow other resolvers to be tried.\n> Otherwise, a `FunctionClauseError` would be raised and the whole build would\n> fail.\n\nAs all defined resolvers will be tried, there is no need call the built-in\nresolvers from your own resolver before running some expensive or slow\ncomputation (such as an HTTP call) because they will be called anyway.\n\n```elixir\n# You may do this\n\ndefmodule MyApp.SchemaResolver do\n   def resolve(\"https://\" <> _ = uri, _opts) do\n    with {:error, {:not_embedded, _}} <- JSV.Resolver.Embedded.resolve(uri, []),\n         {:ok, %{status: 200, body: schema}} <- MyApp.HttpClient.get(uri) do\n      {:ok, schema}\n    end\n  end\n\n  def resolve(_, _) do\n    {:error, :unknown}\n  end\nend\n```\n\nThe built-in resolvers are standard resolvers implementations and adhere to the\n`JSV.Resolver` behaviour. That means that you can just pass them _before_ yours\nas regular resolvers:\n\n```elixir\n# You should do this instead\n\nroot = JSV.build!(schema, resolvers: [JSV.Resolver.Embedded, MyApp.SchemaResolver])\n```\n\nThe `JSV.Resolver.Embedded` resolver will only be called once.\n\nThere may be valid use cases for delegation. If you know of one, just let us\nknow!","ref":"resolvers.html#resolvers-form-a-chain"},{"type":"extras","title":"Vocabularies","doc":"# Vocabularies\n\nJSV follows the JSON Schema Draft 2020-12 specification, which uses the\nvocabulary system to define the capabilities of schemas.\n\nFuture releases of JSV will allow defining custom vocabularies, but for now, JSV\nonly uses a fixed set of schema keyword implementations.","ref":"vocabularies.html"},{"type":"extras","title":"Meta-schemas: Introduction to vocabularies - Vocabularies","doc":">","ref":"vocabularies.html#meta-schemas-introduction-to-vocabularies"},{"type":"extras","title":"Note - Vocabularies","doc":">\n> You can skip this section if you are not interested in the inner workings of\n> the JSON Schema specification.\n\nHere is what happens when validating with the latest specification:","ref":"vocabularies.html#note"},{"type":"extras","title":"The well-known and official schema - Vocabularies","doc":"The\n[`https://json-schema.org/draft/2020-12/schema`](https://json-schema.org/draft/2020-12/schema)\nschema defines the following vocabulary:\n\n```json\n{\n  \"$vocabulary\": {\n    \"https://json-schema.org/draft/2020-12/vocab/core\": true,\n    \"https://json-schema.org/draft/2020-12/vocab/applicator\": true,\n    \"https://json-schema.org/draft/2020-12/vocab/unevaluated\": true,\n    \"https://json-schema.org/draft/2020-12/vocab/validation\": true,\n    \"https://json-schema.org/draft/2020-12/vocab/meta-data\": true,\n    \"https://json-schema.org/draft/2020-12/vocab/format-annotation\": true,\n    \"https://json-schema.org/draft/2020-12/vocab/content\": true\n  }\n}\n```\n\nThe vocabulary is split into different parts, here one by object property.\n\nThe `true` values in the `$vocabulary` keyword do not mean \"enabled\" but\n\"enforced\". A `false` value allows libraries to skip the vocabulary if they do\nnot implement it.\n\nMore information can be found on the [official\nwebsite](https://json-schema.org/learn/glossary#vocabulary).\n\n\n### $schema declaration\n\nThe well-known schema can be used as a meta-schema. This is done by using its\nURI as the `$schema` property of the inheriting schema.\n\nWe would like to use the `type` keyword to validate some data. To let the\nlibrary know what keywords should be handled, our schema declares the\n`https://json-schema.org/draft/2020-12/schema` as its meta-schema using the\n`$schema` keyword.\n\n```json\n  {\n    \"$schema\": \"https://json-schema.org/draft/2020-12/schema\",\n    \"type\": \"integer\"\n  }\n```\n\nJSV will use `https://json-schema.org/draft/2020-12/schema` if the `$schema`\nproperty is not defined in your schemas.\n\nVocabularies have limitations:\n\n* If you declare both `$schema` and `$vocabulary` in your schema, another schema\n  using yours as its meta-schema will only use your `$vocabulary`, and not the\n  one of the referenced `$schema`.\n* You cannot declare a `$vocabulary` property in your schema and expect it to be\n  used and/or override those of the meta-schema. It will only be used by another\n  schema referencing your own as its `$schema`.\n\nIt is a bit like an object oriented class system where you can only call methods\nfrom the parent class but neither from the child class or the parent's parent\nclass.","ref":"vocabularies.html#the-well-known-and-official-schema"},{"type":"extras","title":"Implementation in libraries and tools - Vocabularies","doc":"Libraries such as JSV must map this vocabulary to implementations.\n\nFor instance, in JSV, the\n`https://json-schema.org/draft/2020-12/vocab/validation` part that defines the\n`type` keyword is implemented with the `JSV.Vocabulary.V202012.Validation`\nElixir module.\n\nIf that vocabulary is not declared in your meta-schema, JSV will not use that\nmodule and **the `type` keyword and any other keyword declared by the\n[validation vocabulary](https://json-schema.org/draft/2020-12/vocab/validation)\nwill be ignored during validation**.","ref":"vocabularies.html#implementation-in-libraries-and-tools"},{"type":"extras","title":"Validate Data With Schemas","doc":"# Validate Data With Schemas\n\n\n\nTo validate a term, call the `JSV.validate/3` function like so:\n\n```elixir\nJSV.validate(data, root_schema, opts)\n```\n\nThe return value of `JSV.validate/3` returns cast data. See the documentation of\nthat function for more information.","ref":"validation-basics.html"},{"type":"extras","title":"General considerations - Validate Data With Schemas","doc":"* JSV supports all keywords of the 2020-12 specification except\n  `contentMediaType`, `contentEncoding` and `contentSchema`. They are ignored.\n  Future support for custom vocabularies will allow you to validate data with\n  such keywords.\n* The `format` keyword is largely supported but with many inconsistencies,\n  mostly due to differences between Elixir and JavaScript (JSON Schema is\n  largely based on JavaScript primitives). For most use cases, the differences\n  are negligible.\n* The `\"integer\"` type will transform floats into integer when the fractional\n  part is zero (such as `123.0`). Elixir implementation for floating-point\n  numbers with large integer parts may return incorrect results. Example:\n\n      > trunc(1000000000000000000000000.0)\n      # ==>    999999999999999983222784\n\n  When dealing with such data it may be better to discard the cast data, or to\n  work with strings instead of floats.","ref":"validation-basics.html#general-considerations"},{"type":"extras","title":"Formats - Validate Data With Schemas","doc":"JSV supports multiple formats out of the box with its default implementation,\nbut some are only available under certain conditions that will be specified for\neach format.\n\nThe following listing describes the condition for support and return value type\nfor these default implementations. You can override those implementations by\nproviding your own, as well as providing new formats. This will be described\nlater in this document.\n\nAlso, note that by default, JSV format validation will return the original\nvalue, that is, the string form of the data. Some format validators can also\ncast the string to a more interesting data structure, for instance converting a\ndate string to a `Date` struct. You can enable returning specific format cast\nvalues by passing the `cast_formats: true` option to `JSV.validate/3`.\n\nThe listing below describe values returned when that option is enabled.\n\n**Important**: Many formats require the `abnf_parsec` library to be available.\n\nYou may add this dependency in your application and it will be used\nautomatically.\n\n```elixir\ndef deps do\n  [\n    {:abnf_parsec, \"~> 2.0\"},\n  ]\nend\n```\n\n<!-- rdmx jsv:formats -->","ref":"validation-basics.html#formats"},{"type":"extras","title":"date - Validate Data With Schemas","doc":"* **support**: Native.\n* **input**: `\"2020-04-22\"`\n* **output**: `~D[2020-04-22]`\n* The format is implemented with the native `Date` module.\n* The native `Date` module supports the `YYYY-MM-DD` format only. `2024`, `2024-W50`, `2024-12` will not be valid.","ref":"validation-basics.html#date"},{"type":"extras","title":"date-time - Validate Data With Schemas","doc":"* **support**: Native.\n* **input**: `\"2025-01-02T00:11:23.416689Z\"`\n* **output**: `~U[2025-01-02 00:11:23.416689Z]`\n* The format is implemented with the native `DateTime` module.\n* The native `DateTime` module supports the `YYYY-MM-DD` format only for dates. `2024T...`, `2024-W50T...`, `2024-12T...` will not be valid.\n* Decimal precision is not capped to milliseconds. `2024-12-14T23:10:00.500000001Z` will be valid.","ref":"validation-basics.html#date-time"},{"type":"extras","title":"duration - Validate Data With Schemas","doc":"* **support**: Requires Elixir 1.17\n* **input**: `\"P1DT4,5S\"`\n* **output**: `%Duration{day: 1, second: 4, microsecond: {500000, 1}}`\n* The format is implemented with the native `Duration` module.\n* Elixir documentation states that _Only seconds may be specified with a decimal fraction, using either a comma or a full stop: P1DT4,5S_.\n* Elixir durations accept negative values.\n* Elixir durations accept out-of-range values, for instance more than 59 minutes.\n* Excessive precision (as in `\"PT10.0000000000001S\"`) will be valid.","ref":"validation-basics.html#duration"},{"type":"extras","title":"email - Validate Data With Schemas","doc":"* **support**: Requires `{:abnf_parsec, \"~> 2.0\"}`.\n* **input**: `\"hello@json-schema.org\"`\n* **output**: Input value.\n* Support is limited by the implementation of that library.\n* The `idn-email` format is not supported out-of-the-box.","ref":"validation-basics.html#email"},{"type":"extras","title":"hostname - Validate Data With Schemas","doc":"* **support**: Native.\n* **input**: `\"some-host\"`\n* **output**: Input value.\n* The format is implemented with the native `Regex` module.\n* Accepts numerical TLDs and single letter TLDs.\n* Uses this regular expression: `^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\\-]*[a-zA-Z0-9])\\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\\-]*[A-Za-z0-9])$` ( Regexper ).\n* Length of components is not checked. Some hostnames could exceed the allowed length.","ref":"validation-basics.html#hostname"},{"type":"extras","title":"ipv4 - Validate Data With Schemas","doc":"* **support**: Native.\n* **input**: `\"127.0.0.1\"`\n* **output**: `{127, 0, 0, 1}`\n* The format is implemented with the native `:inet` module.","ref":"validation-basics.html#ipv4"},{"type":"extras","title":"ipv6 - Validate Data With Schemas","doc":"* **support**: Native.\n* **input**: `\"::1\"`\n* **output**: `{0, 0, 0, 0, 0, 0, 0, 1}`\n* The format is implemented with the native `:inet` module.","ref":"validation-basics.html#ipv6"},{"type":"extras","title":"iri - Validate Data With Schemas","doc":"* **support**: Requires `{:abnf_parsec, \"~> 2.0\"}`.\n* **input**: `\"https://héhé.com/héhé\"`\n* **output**: `%URI{scheme: \"https\", authority: \"héhé.com\", userinfo: nil, host: \"héhé.com\", port: 443, path: \"/héhé\", query: nil, fragment: nil}`","ref":"validation-basics.html#iri"},{"type":"extras","title":"iri-reference - Validate Data With Schemas","doc":"* **support**: Requires `{:abnf_parsec, \"~> 2.0\"}`.\n* **input**: `\"//héhé\"`\n* **output**: `%URI{scheme: nil, authority: \"héhé\", userinfo: nil, host: \"héhé\", port: nil, path: nil, query: nil, fragment: nil}`","ref":"validation-basics.html#iri-reference"},{"type":"extras","title":"json-pointer - Validate Data With Schemas","doc":"* **support**: Requires `{:abnf_parsec, \"~> 2.0\"}`.\n* **input**: `\"/foo/bar/baz\"`\n* **output**: Input value.","ref":"validation-basics.html#json-pointer"},{"type":"extras","title":"regex - Validate Data With Schemas","doc":"* **support**: Native.\n* **input**: `\"[a-zA-Z0-9]\"`\n* **output**: `~r/[a-zA-Z0-9]/`\n* The format is implemented with the native `Regex` module.\n* The `Regex` module does not follow the `ECMA-262` specification.","ref":"validation-basics.html#regex"},{"type":"extras","title":"relative-json-pointer - Validate Data With Schemas","doc":"* **support**: Requires `{:abnf_parsec, \"~> 2.0\"}`.\n* **input**: `\"0/foo/bar\"`\n* **output**: Input value.","ref":"validation-basics.html#relative-json-pointer"},{"type":"extras","title":"time - Validate Data With Schemas","doc":"* **support**: Native.\n* **input**: `\"20:20:08.378586\"`\n* **output**: `~T[20:20:08.378586]`\n* The format is implemented with the native `Time` module.\n* The native `Time` implementation will completely discard the time offset information. Invalid offsets will be valid.\n* Decimal precision is not capped to milliseconds. `23:10:00.500000001` will be valid.","ref":"validation-basics.html#time"},{"type":"extras","title":"unknown - Validate Data With Schemas","doc":"* **support**: Native\n* **input**: `\"anything\"`\n* **output**: Input value.\n* No validation or transformation is done.","ref":"validation-basics.html#unknown"},{"type":"extras","title":"uri - Validate Data With Schemas","doc":"* **support**: Native, optionally uses `{:abnf_parsec, \"~> 2.0\"}`.\n* **input**: `\"http://example.com\"`\n* **output**: `%URI{scheme: \"http\", authority: \"example.com\", userinfo: nil, host: \"example.com\", port: 80, path: nil, query: nil, fragment: nil}`\n* The format is implemented with the native `URI` module.\n* Without the optional dependency, the `URI` module is used and a minimum checks on hostname and scheme presence are made.","ref":"validation-basics.html#uri"},{"type":"extras","title":"uri-reference - Validate Data With Schemas","doc":"* **support**: Native, optionally uses `{:abnf_parsec, \"~> 2.0\"}`.\n* **input**: `\"/example-path\"`\n* **output**: `%URI{scheme: nil, userinfo: nil, host: nil, port: nil, path: \"/example-path\", query: nil, fragment: nil}`\n* The format is implemented with the native `URI` module.\n* Without the optional dependency, the `URI` module will cast most non url-like strings as a `path`.","ref":"validation-basics.html#uri-reference"},{"type":"extras","title":"uri-template - Validate Data With Schemas","doc":"* **support**: Requires `{:abnf_parsec, \"~> 2.0\"}`.\n* **input**: `\"http://example.com/search{?query,lang}\"`\n* **output**: Input value.","ref":"validation-basics.html#uri-template"},{"type":"extras","title":"uuid - Validate Data With Schemas","doc":"* **support**: Native\n* **input**: `\"bf22824c-c8a4-11ef-9642-0fdaf117eeb9\"`\n* **output**: Input value.\n\n<!-- rdmx /jsv:formats -->","ref":"validation-basics.html#uuid"},{"type":"extras","title":"Custom formats - Validate Data With Schemas","doc":"In order to provide custom formats, or to override default implementations for\nformats, you may provide a list of modules as the value for the `:formats`\noptions of `JSV.build/2`. Such modules must implement the `JSV.FormatValidator`\nbehaviour.","ref":"validation-basics.html#custom-formats"},{"type":"extras","title":"Example - Validate Data With Schemas","doc":"```elixir\ndefmodule CustomFormats do\n  @behaviour JSV.FormatValidator\n\n  @impl true\n  def supported_formats do\n    [\"greeting\"]\n  end\n\n  @impl true\n  def validate_cast(\"greeting\", data) do\n    case data do\n      \"hello \" <> name -> {:ok, %Greeting{name: name}}\n      _ -> {:error, :invalid_greeting}\n    end\n  end\nend\n```\n\nWith this module you can now call the builder with it:\n\n```elixir\nJSV.build!(raw_schema, formats: [CustomFormats])\n```\n\nNote that this will disable all other formats. If you need to still support the\ndefault formats, a helper is available:\n\n```elixir\nJSV.build!(raw_schema,\n  formats: [CustomFormats | JSV.default_format_validator_modules()]\n)\n```\n\nFormat validation modules are checked during the build phase, in order. So you\ncan override any format defined by a module that comes later in the list,\nincluding the default modules.","ref":"validation-basics.html#example"},{"type":"extras","title":"Decimal Support","doc":"# Decimal Support\n\nJSV supports validation of Decimal values from the [Decimal](https://hex.pm/packages/decimal) library.\n\nData with decimal values can be obtained at the JSON parsing level, which is not\nhandled by JSV. For instance with `Jason.decode!(json, floats: :decimals)`.\n\nDecimal values in schemas are not supported.","ref":"decimal-support.html"},{"type":"extras","title":"Supported Validations - Decimal Support","doc":"The following validations work when the data is a `Decimal` struct:\n\n- `type`: supports both `:number` and `:integer` types\n  - `:number` accepts any `Decimal` struct.\n  - `:integer` accepts only integer `Decimal` structs and converts them to\n    Elixir integers.\n- `maximum`: validates that a Decimal value is less than or equal to the maximum.\n- `exclusiveMaximum`: validates that a Decimal value is strictly less than the\n  maximum.\n- `minimum`: validates that a Decimal value is greater than or equal to the\n  minimum.\n- `exclusiveMinimum`: validates that a Decimal value is strictly greater than\n  the minimum.\n- `multipleOf`: validates that a Decimal value is a multiple of the given\n  number.","ref":"decimal-support.html#supported-validations"},{"type":"extras","title":"Examples - Decimal Support","doc":"When validating integers, decimal values are replaced with integers.\n\n<!-- rdmx :section name:example_integer format:true -->\n```elixir\nschema = %{\n  type: :array,\n  items: %{type: :integer}\n}\n\nroot = JSV.build!(schema)\n\n# 4.00001 is not an integer\n{:error, _} = JSV.validate([3, Decimal.new(\"4.00001\")], root)\nJSV.validate([1, Decimal.new(\"2.0\")], root)\n```\n<!-- rdmx /:section -->\n<!-- rdmx :eval section:example_integer -->\n```elixir\n{:ok, [1, 2]}\n```\n<!-- rdmx /:eval -->\n\nWhen validating numbers, decimal values with zero decimal part are preserved.\n<!-- rdmx :section name:example_number format:true -->\n```elixir\nschema = %{\n  type: :array,\n  items: %{type: :number}\n}\n\nroot = JSV.build!(schema)\n\ndata =\n  Jason.decode!(\n    \"\"\"\n    [1, 2.0, 3, 4.00001]\n    \"\"\",\n    floats: :decimals\n  )\n\nJSV.validate(data, root)\n```\n<!-- rdmx /:section -->\n<!-- rdmx :eval section:example_number -->\n```elixir\n{:ok, [1, Decimal.new(\"2.0\"), 3, Decimal.new(\"4.00001\")]}\n```\n<!-- rdmx /:eval -->","ref":"decimal-support.html#examples"},{"type":"extras","title":"Limitations - Decimal Support","doc":"The following validations do not support nested Decimal values:\n\n- `uniqueItems`: arrays containing Decimal values.\n- `const`: when the constant value contains Decimal values.\n- `enum`: when the enum values contain Decimal values.\n\nThis is because while we could make a simple check for values that are just a\ndecimal struct, those keywords can validate arbitrarily nested data.\n\nJSV will not perform a deep comparison between data and a `const` or `enum`\nvalue, or between items of an array when using `uniqueItems`, but rather just\nuse the `==` operator for speed.\n\nWe believe that handling those cases is out of JSV's scope, as enums and consts\nare most often used with strings or atoms representing states, roles,\ncategories, tags, _etc._\n\nCustom vocabularies can be added to implement deep matching with support for\ndecimal comparisons.","ref":"decimal-support.html#limitations"},{"type":"extras","title":"API Changes in JSV 0.9","doc":"# API Changes in JSV 0.9\n\nJSV 0.9 introduces significant changes to the schema building API. While these\nchanges are designed to improve the long-term maintainability and usability of\nthe library, they do deprecate some existing functions. This guide explains the\nchanges, their rationale, and how to migrate your code.","ref":"api-changes-v0-9.html"},{"type":"extras","title":"Overview of Changes - API Changes in JSV 0.9","doc":"The main changes in JSV 0.9 are:\n\n1. **Deprecation of the old composition API**: Functions returning predefined\n   schemas in the `JSV.Schema` module like `integer/0` or `all_of/1` are now\n   deprecated.\n2. **Introduction of the composer**: Deprecated functions have been copied to\n   `JSV.Schema.Composer` for backward compatibility.\n3. **New preset functions**: A new `JSV.Schema.Helpers` module provides helper\n   functions that are not composable but rather accept other schema attributes\n   as the last argument. They return maps instead of `%JSV.Struct{}` schemas in\n   all cases.\n4. **Enhanced developer experience**: `use JSV.Schema` now imports all necessary\n   functions to define schemas in plain Elixir code.","ref":"api-changes-v0-9.html#overview-of-changes"},{"type":"extras","title":"The Old Composition API and Its Limitations - API Changes in JSV 0.9","doc":"The previous API allowed for a fluent, pipeline-based approach to schema building but with too many quirks.\n\n<!-- rdmx :section name:old_api format:true -->\n```elixir\nimport JSV.Schema\n\nschema =\n  %Schema{}\n  |> object()\n  |> properties(%{\n    age: integer(description: \"The age\"),\n    name: string(),\n    role:\n      JSV.Schema.string_to_atom_enum(\n        %{description: \"The user role\"},\n        [:admin, :author, :editor]\n      )\n  })\n  |> required([:age, :name])\n```\n<!-- rdmx /:section -->\n\nWhile this approach looked elegant, it had several practical limitations:\n\n1. **Unnecessary abstraction**: If you already know the structure of your\n   schema, there's no need to pay the cost for merging schemas on every function\n   call.\n2. **Awkward argument order**: Functions like\n   `JSV.Schema.Composer.string_to_atom_enum/2` required the base schema as the\n   first argument, making composition cumbersome.\n3. **Overriding composability**: Merging a schema multiple times could lead to\n   overwrite previoulsy defined attributes.\n4. **Unclear cast to struct**: The composition API is designed to work aroun\n   `JSV.Schema.merge/2` that follows complicated rules regarding the base schema\n   to merge into. It will keep maps as-is but will transform `nil` or keyword\n   lists into a `%JSV.Schema{}` struct. This struct has always been designed to\n   support autocompletion only but the compisition API enforces usage of this\n   struct.","ref":"api-changes-v0-9.html#the-old-composition-api-and-its-limitations"},{"type":"extras","title":"The New Approach - API Changes in JSV 0.9","doc":"","ref":"api-changes-v0-9.html#the-new-approach"},{"type":"extras","title":"Direct Schema Definition - API Changes in JSV 0.9","doc":"First, it has awlays been possible to define schemas statically. While more\nverbose, it maps directly to the actual schema that is being defined and allows\nto work with a single shape in mind.\n\n<!-- rdmx :section name:no_funs format:true -->\n```elixir\nschema = %{\n  type: :object,\n  properties: %{\n    age: %{type: :integer, description: \"The age\"},\n    name: %{type: :string}\n  },\n  required: [:age, :name]\n}\n```\n<!-- rdmx /:section -->\n\nThis approach is more explicit and performant, and doesn't require intermediate\npipeline steps when the final structure is known. No function calls means no\nunexpected attributes.","ref":"api-changes-v0-9.html#direct-schema-definition"},{"type":"extras","title":"New Helper Functions with Improved API - API Changes in JSV 0.9","doc":"The new `JSV.Schema.Helpers` module provides functions with a more intuitive\nargument order.\n\nWe also took the liberty to change some function names like\n`string_enum_to_atom` instead of `string_to_atom_enum`.\n\n<!-- rdmx :section name:enum_example format:true -->\n```elixir\nschema = %{\n  properties: %{\n    age: integer(description: \"The age\"),\n    name: string(),\n    role:\n      JSV.Schema.Helpers.string_enum_to_atom(\n        [:admin, :author, :editor],\n        description: \"Some description\"\n      )\n  }\n}\n```\n<!-- rdmx /:section -->\n\nThe key improvement is that the primary argument comes first (the enum values in\nthis example), and additional attributes are provided as a list or map at the\nend.","ref":"api-changes-v0-9.html#new-helper-functions-with-improved-api"},{"type":"extras","title":"Helpers do not return JSV.Schema structs - API Changes in JSV 0.9","doc":"As visible above, helpers that do not take any primary argument look the same in\nboth APIs. While there is no visible change in the calls, there is an important\ndifference:\n\n**This new API does not enforce creation of a `%JSV.Schema{}` struct** and will return bare maps instead. This allows to work with user-defined schema vocabularies using keywords that are now known in the struct.\n\nSo `integer(description: \"some int\")` will now return:\n\n```elixir\n%{type: :integer, description: \"some int\"}\n```\n\nWhere the composition API would return:\n\n```elixir\n%JSV.Schema{type: :integer, description: \"some int\"}\n```","ref":"api-changes-v0-9.html#helpers-do-not-return-jsv-schema-structs"},{"type":"extras","title":"Composing with the new API - API Changes in JSV 0.9","doc":"We suggest to simply use `Map.put/3` and `Map.merge/2` to build schemas\ndynamically. It's less obscure and allows you to build your own helpers on top\nof it.\n\nA new `JSV.Schema.Helpers.~>/2` operator is available and brings back the composition API on top of the new helpers:\n\n<!-- rdmx :section name:new_op format:true -->\n```elixir\nobject(description: \"a user\")\n~> any_of([AdminSchema, CustomerSchema])\n~> properties(foo: integer())\n```\n<!-- rdmx /:section -->","ref":"api-changes-v0-9.html#composing-with-the-new-api"},{"type":"extras","title":"A new use macro to define schemas. - API Changes in JSV 0.9","doc":"You can now import all necessary schema-defining functions with a single `use`\nstatement.\n\nThis will import macros (`JSV.defschema/1`, `JSV.defcast/{1,2,3}`) from `JSV`\nand the new preset functions including the `JSV.Schema.Helpers.sigil_SD/2`\nsigil.\n\n<!-- rdmx :section name:use_example format:true -->\n```elixir\ndefmodule MySchemas do\n  use JSV.Schema\n\n  defschema %{\n    type: :object,\n    description: ~SD\"\"\"\n    This description spans multiple lines for readability.\n\n    But the sigil will make it a oneliner.\n    \"\"\",\n    properties: %{\n      id: integer(minimum: 1),\n      name: string(minLength: 1),\n      email: string(format: \"email\"),\n      role: string_enum_to_atom([:admin, :user, :guest])\n    },\n    required: [:id, :name, :email, :role]\n  }\nend\n```\n<!-- rdmx /:section -->","ref":"api-changes-v0-9.html#a-new-use-macro-to-define-schemas"},{"type":"extras","title":"Future Considerations - API Changes in JSV 0.9","doc":"","ref":"api-changes-v0-9.html#future-considerations"},{"type":"extras","title":"Backward Compatibility Timeline - API Changes in JSV 0.9","doc":"- **`JSV.Schema.Composer`**: This module is maintained for backward compatibility only. No new functions will be added to it in future versions.\n- **`JSV.Schema.Helpers`**: This is the new home for helper functions and will continue to grow with new presets and utilities.","ref":"api-changes-v0-9.html#backward-compatibility-timeline"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}